import pytest

def test_ise():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['vlogdefine', 'vlogparam']
    name         = 'test_ise_0'
    tool         = 'ise'
    tool_options = {
        'family'  : 'spartan6',
        'device'  : 'xc6slx45',
        'package' : 'csg324',
        'speed'   : '-2'
    }

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options)
    backend.configure(args)

    compare_files(ref_dir, work_root, ['Makefile',
                                       'config.mk',
                                       name+'.tcl',
                                       name+'_run.tcl',
    ])

    #f = os.path.join(work_root, 'pcf_file.pcf')
    #with open(f, 'a'):
    #    os.utime(f, None)

    backend.build()
    compare_files(ref_dir, work_root, ['xtclsh.cmd'])

def test_ise_missing_options():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['vlogdefine', 'vlogparam']
    name         = 'test_ise_0'
    tool         = 'ise'
    tool_options = {
        'family'  : 'spartan6',
        'device'  : 'xc6slx45',
        'package' : 'csg324',
    }

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options)
    with pytest.raises(RuntimeError) as e:
        backend.configure('')
    assert "Missing required option 'speed'" in str(e.value)



import logging
import os
import shutil
import yaml
logger = logging.getLogger(__name__)

class Edalizer(object):

    def __init__(self, vlnv, flags, cores, work_root, export_root=None):
        if os.path.exists(work_root):
            for f in os.listdir(work_root):
                if os.path.isdir(os.path.join(work_root, f)):
                    shutil.rmtree(os.path.join(work_root, f))
                else:
                    os.remove(os.path.join(work_root, f))
        else:
            os.makedirs(work_root)

        logger.debug("Building EDA API")
        def merge_dict(d1, d2):
            for key, value in d2.items():
                if isinstance(value, dict):
                    d1[key] = merge_dict(d1.get(key, {}), value)
                elif isinstance(value, list):
                    d1[key] = d1.get(key, []) + value
                else:
                    d1[key] = value
            return d1

        files        = []
        parameters   = {}
        scripts      = {}
        tool_options = {}
        vpi          = []

        _flags = flags.copy()
        for core in cores:
            logger.info("Preparing " + str(core.name))
            core.setup()

            logger.debug("Collecting EDA API parameters from {}".format(str(core.name)))
            _flags['is_toplevel'] = (core.name == vlnv)

            #Extract files
            if export_root:
                files_root = os.path.join(export_root, core.sanitized_name)
                core.export(files_root, _flags)
            else:
                files_root = core.files_root

            rel_root = os.path.relpath(files_root, work_root)

            #Extract parameters
            merge_dict(parameters, core.get_parameters(_flags))

            #Extract tool options
            merge_dict(tool_options, core.get_tool_options(_flags))

            #Extract scripts
            merge_dict(scripts, core.get_scripts(rel_root, _flags))

            for file in core.get_files(_flags):
                if file.copyto:
                    _name = file.copyto
                    dst = os.path.join(work_root, _name)
                    _dstdir = os.path.dirname(dst)
                    if not os.path.exists(_dstdir):
                        os.makedirs(_dstdir)
                    shutil.copy2(os.path.join(files_root, file.name),
                                 dst)
                else:
                    _name = os.path.join(rel_root, file.name)
                files.append({
                    'name'            : _name,
                    'file_type'       : file.file_type,
                    'is_include_file' : file.is_include_file,
                    'logical_name'    : file.logical_name})
            #Extract VPI modules
            for _vpi in core.get_vpi(_flags):
                vpi.append({'name'         : _vpi['name'],
                            'src_files'    : [os.path.join(rel_root, f) for f in _vpi['src_files']],
                            'include_dirs' : [os.path.join(rel_root, i) for i in _vpi['include_dirs']],
                            'libs'         : _vpi['libs']})

        top_core = cores[-1]
        self.edalize = {
            'version'      : '0.2.0',
            'files'        : files,
            'hooks'        : scripts,
            'name'         : top_core.sanitized_name,
            'parameters'   : parameters,
            'tool_options' : {flags['tool'] : tool_options},
            'toplevel'     : top_core.get_toplevel(flags),
            'vpi'          : vpi,
        }

    def to_yaml(self, edalize_file):
        with open(edalize_file,'w') as f:
            f.write(yaml.dump(self.edalize))
        




import pytest
import os.path
tests_dir = os.path.dirname(__file__)

def test_capi2_export():
    import os
    import tempfile
    from fusesoc.core import Core

    core_file = os.path.join(tests_dir,
                             "capi2_cores",
                             "misc",
                             "files.core")
    core = Core(core_file, None, None)

    export_root  = tempfile.mkdtemp(prefix='capi2_export_')

    core.export(export_root)

    expected = [
        'dontpickthisfile',
        'dummy.tcl',
        'scriptfile',
        'subdir/dummy.extra',
        'vhdlfile',
        'vlogfile',
        'vpifile']
    result = []

    for root, dirs, files in os.walk(export_root):
        result += [os.path.relpath(os.path.join(root, f), export_root) for f in files]

    assert expected == sorted(result)

    with pytest.raises(RuntimeError) as excinfo:
        core.export(export_root, {'target' : 'will_fail', 'is_toplevel' : True})
    assert "Cannot find idontexist in" in str(excinfo.value)

    core.files_root = os.path.join(tests_dir, __name__)
    core.export(export_root, {'target' : 'files_root_test', 'is_toplevel' : True})
    expected = ['targets.info']

    result = []

    for root, dirs, files in os.walk(export_root):
        result += [os.path.relpath(os.path.join(root, f), export_root) for f in files]
    assert expected == sorted(result)

def test_capi2_get_depends():
    from fusesoc.core import Core
    from fusesoc.vlnv import Vlnv

    core = Core(os.path.join(tests_dir,
                             "capi2_cores",
                             "misc",
                             "depends.core"))
    flags = {}
    result = core.get_depends(flags)

    expected = [
        Vlnv('unversioned'),
        Vlnv('versioned-1.0'),
        Vlnv('<lt-1.0'),
        Vlnv('<=lte-1.0'),
        Vlnv('=eq-1.0'),
        Vlnv('>gt-1.0'),
        Vlnv('>=gte-1.0'),
        Vlnv('::n'),
        Vlnv('::nv:1.0'),
        Vlnv(':l:nv:1.0'),
        Vlnv('v:l:nv:1.0'),
        Vlnv('<::vlnvlt:1.0'),
        Vlnv('<=::vlnvlte:1.0'),
        Vlnv('=::vlnveq:1.0'),
        Vlnv('>::vlnvgt:1.0'),
        Vlnv('>=::vlnvgte:1.0'),
    ]
    assert len(result) == len(expected)
    for i in range(len(result)):
        assert result[i] == expected[i]

def test_capi2_get_files():
    from fusesoc.core import Core


    core_file = os.path.join(tests_dir,
                             "capi2_cores",
                             "misc",
                             "files.core")
    core = Core(core_file, None, None)

    expected =  [
        {'is_include_file' : False,
         'file_type'       : 'user',
         'copyto'          : 'copied.file',
         'logical_name'    : '',
         'name'            : 'subdir/dummy.extra'},
        {'is_include_file' : False,
         'file_type'       : 'tclSource',
         'copyto'          : 'subdir/another.file',
         'logical_name'    : '',
         'name'            : 'dummy.tcl'},
        {'copyto'          : '',
         'file_type'       : 'verilogSource',
         'is_include_file' : True,
         'logical_name'    : '',
         'name'            : 'vlogfile'},
        {'copyto'          : '',
         'file_type'       : 'vhdlSource',
         'is_include_file' : False,
         'logical_name'    : '',
         'name'            : 'vhdlfile'},
        {'copyto'          : '',
         'file_type'       : 'user',
         'is_include_file' : False,
         'logical_name'    : '',
         'name'            : 'pickthisfile'},
        ]

    flags = {'tool' : 'icarus'}
    result = [vars(x) for x in core.get_files(flags)]
    assert expected == result

def test_capi2_get_parameters():
    from fusesoc.core import Core

    core_file = os.path.join(tests_dir,
                             "capi2_cores",
                             "misc",
                             "parameters.core")
    core = Core(core_file)

    param1 = {
        'datatype'  : 'str',
        'paramtype' : 'vlogparam',
    }
    param2 = {
        'datatype'    : 'str',
        'default'     : 'default_value',
        'description' : 'This is a parameter',
        'paramtype'   : 'vlogparam',
        }
    intparam = {
        'datatype'  : 'int',
        'default'   : 5446,
        'paramtype' : 'vlogparam',
    }
    boolfalse = {
        'datatype'  : 'bool',
        'paramtype' : 'vlogparam',
    }

    booltrue = {
        'datatype'  : 'bool',
        'default'   : True,
        'paramtype' : 'vlogparam',
    }
    flags    = {'is_toplevel' : True}
    expected = {'param1' : param1}

    assert expected == core.get_parameters(flags)

    flags['target'] = 'noparameters'
    expected = {}
    assert expected == core.get_parameters(flags)

    flags['target'] = 'nonexistant'
    with pytest.raises(SyntaxError) as excinfo:
        core.get_parameters(flags)
    assert "Parameter 'idontexist', requested by target 'nonexistant', was not found" in str(excinfo.value)

    flags['target'] = 'multiparameters'
    expected = {'param1' : param1, 'param2' : param2}
    assert expected == core.get_parameters(flags)

    flags['target'] = 'use_flags'
    expected = {'param2' : param2}
    assert expected == core.get_parameters(flags)

    flags['tool'] = 'icarus'
    expected = {'param1' : param1}
    assert expected == core.get_parameters(flags)

    flags['target'] = 'types'
    expected = {'param2'    : param2,
                'intparam'  : intparam,
                'boolfalse' : boolfalse,
                'booltrue'  : booltrue,
    }
    assert expected == core.get_parameters(flags)

    flags['target'] = 'override'
    param1['default'] = 'def'
    param2['default'] = 'new_def'
    intparam['default'] = 0xdeadbeef
    boolfalse['default'] = True
    booltrue['default'] = False
    expected = {'param1'    : param1,
                'param2'    : param2,
                'intparam'  : intparam,
                'boolfalse' : boolfalse,
                'booltrue'  : booltrue,
    }
    assert expected == core.get_parameters(flags)
def test_capi2_get_scripts():
    from fusesoc.core import Core

    simple1  = {'cmd' : ['simple_cmd1'],
                'env' : {'FILES_ROOT' : 'my_files_root'},
                'name' : 'simple1'}
    simple2  = {'cmd'  : ['simple_cmd2'],
                'env' : {'FILES_ROOT' : 'my_files_root'},
                'name' : 'simple2'}
    simple3  = {'cmd'  : ['simple_cmd3'],
                'env' : {'FILES_ROOT' : 'my_files_root'},
                'name' : 'simple3'}
    simple4  = {'cmd'  : ['simple_cmd4'],
                'env' : {'FILES_ROOT' : 'my_files_root'},
                'name' : 'simple4'}
    with_env = {'cmd'  : ['simple5'],
                'env'  : {'FILES_ROOT' : 'my_files_root',
                          'TESTENV' : 'testvalue'},
                'name' : 'with_env'}
    multi_cmd = {'cmd'  : ['command', 'with', 'args'],
                 'env' : {'FILES_ROOT' : 'my_files_root'},
                 'name' : 'multi_cmd'}
    core_file = os.path.join(tests_dir,
                             "capi2_cores",
                             "misc",
                             "hooks.core")
    core = Core(core_file, None, None)

    flags    = {'is_toplevel' : True}
    expected = {'pre_build' : [simple1]}
    assert expected == core.get_scripts("my_files_root", flags)

    flags['target'] = 'nohooks'
    expected = {}
    assert expected == core.get_scripts("my_files_root", flags)

    flags['target'] = 'nonexistant'
    with pytest.raises(SyntaxError) as excinfo:
        core.get_scripts("", flags)
    assert "Script 'idontexist', requested by target 'nonexistant', was not found" in str(excinfo.value)

    flags['target'] = 'allhooks'
    expected = {
        'pre_build'  : [simple1],
        'post_build' : [simple2],
        'pre_run'    : [simple3],
        'post_run'   : [simple4],
        }
    assert expected == core.get_scripts("my_files_root", flags)

    flags['target'] = 'multihooks'
    expected = {
        'pre_run'  : [simple1,
                      with_env,
                      multi_cmd]
        }
    assert expected == core.get_scripts("my_files_root", flags)

    flags['target'] = 'use_flags'
    expected = {'post_run' : [simple2]}
    assert expected == core.get_scripts("my_files_root", flags)

    flags['tool'] = 'icarus'
    expected = {'post_run' : [simple1]}
    assert expected == core.get_scripts("my_files_root", flags)

def test_capi2_get_tool_options():
    from fusesoc.core import Core

    core_file = os.path.join(tests_dir,
                             "capi2_cores",
                             "misc",
                             "targets.core")
    core = Core(core_file, None, None)

    with pytest.raises(KeyError):
        core.get_tool_options({})

    assert {} == core.get_tool_options({'tool' : 'icarus'})
    flags = {'target'      : 'target_with_tool_options',
             'is_toplevel' : True}

    flags['tool'] = 'icarus'
    expected = {'iverilog_options' : ['a', 'few', 'options']}
    assert expected == core.get_tool_options(flags)

    flags['tool'] = 'vivado'
    expected = {'part' : 'xc7a35tcsg324-1'}
    assert expected == core.get_tool_options(flags)

    flags['tool'] = 'invalid'
    expected = {}
    assert expected == core.get_tool_options(flags)

def test_capi2_get_toplevel():
    from fusesoc.core import Core

    core_file = os.path.join(tests_dir,
                             "capi2_cores",
                             "misc",
                             "toplevel.core")
    core = Core(core_file)

    flags = {'target' : 'no_toplevel'}
    with pytest.raises(SyntaxError):
        core.get_toplevel(flags)

    flags = {'target' : 'str_toplevel'}
    assert 'toplevel_as_string'  == core.get_toplevel(flags)

    flags = {'target' : 'list_toplevel'}
    assert 'toplevel as list'  == core.get_toplevel(flags)

def test_capi2_get_vpi():
    from fusesoc.core import Core

    core_file = os.path.join(tests_dir,
                             "capi2_cores",
                             "misc",
                             "vpi.core")
    core = Core(core_file)

    expected = [
        {'src_files': ['f1',
                       'f3'],
         'include_dirs': [''],
         'libs': ['some_lib'],
         'name': 'vpi1'},
        {'src_files': ['f4'],
         'include_dirs': [],
         'libs': [],
         'name': 'vpi2'}
    ]

    assert [] == core.get_vpi({'is_toplevel' : True, 'target' : 'invalid'})
    assert expected == core.get_vpi({'is_toplevel' : True})
    assert expected == core.get_vpi({'is_toplevel' : False})

def test_capi2_get_work_root():
    from fusesoc.core import Core

    core_file = os.path.join(tests_dir,
                             "capi2_cores",
                             "misc",
                             "targets.core")
    core = Core(core_file, None, None)

    with pytest.raises(KeyError):
        core.get_work_root({})
    assert 'default-icarus'      == core.get_work_root({'tool' : 'icarus'})
    assert 'default-vivado'      == core.get_work_root({'tool' : 'vivado'})
    assert 'default-icarus'      == core.get_work_root({'tool' : 'icarus',
                                                        'target' : None})
    with pytest.raises(SyntaxError):
        core.get_work_root({'tool' : 'icarus', 'target' : 'invalid_target'})
    assert 'empty_target-icarus' == core.get_work_root({'tool' : 'icarus', 'target' : 'empty_target'})

def test_capi2_info():
    from fusesoc.core import Core
    for core_name in ['targets']:
        core_file = os.path.join(tests_dir,
                                 "capi2_cores",
                                 "misc",
                                 core_name+'.core')
        core = Core(core_file, None, None)

        gen_info = '\n'.join([x for x in core.info().split('\n') if not 'Core root' in x])
        with open(os.path.join(tests_dir, __name__, core_name+".info")) as f:
            assert f.read() == gen_info, core_name



import difflib
import os
import pytest

from fusesoc.core import Core

def compare_fileset(fileset, name, files):
    assert name == fileset.name
    for i in range(len(files)):
        assert files[i] == fileset.file[i].name

def test_core_info():
    tests_dir = os.path.dirname(__file__)
    cores_root = os.path.join(tests_dir, 'cores')
    for core_name in ['sockit', 'mor1kx-generic']:
        core = Core(os.path.join(cores_root, core_name, core_name+'.core'))
        gen_info = '\n'.join([x for x in core.info().split('\n') if not 'Core root' in x])
        with open(os.path.join(tests_dir, __name__, core_name+".info")) as f:
            assert f.read() == gen_info, core_name

def test_core_parsing():
    from fusesoc.vlnv import Vlnv

    cores_root = os.path.join(os.path.dirname(__file__), 'cores', 'misc')
    core = Core(os.path.join(cores_root, 'nomain.core'))
    assert core.name == Vlnv("::nomain:0")

    import sys
    if sys.version_info[0] > 2:
        with pytest.raises(SyntaxError) as e:
            core = Core(os.path.join(cores_root, "duplicateoptions.core"))
        assert "option 'file_type' in section 'fileset dummy' already exists" in str(e.value)

def test_capi1_get_parameters():
    tests_dir = os.path.join(os.path.dirname(__file__),
                             __name__)

    with pytest.raises(SyntaxError) as e:
        core = Core(os.path.join(tests_dir, 'parameters_nodatatype.core'))
    assert "Invalid datatype '' for parameter" in str(e.value)

    with pytest.raises(SyntaxError) as e:
        core = Core(os.path.join(tests_dir, 'parameters_invaliddatatype.core'))
    assert "Invalid datatype 'badtype' for parameter" in str(e.value)

    with pytest.raises(SyntaxError) as e:
        core = Core(os.path.join(tests_dir, 'parameters_noparamtype.core'))
    assert "Invalid paramtype '' for parameter" in str(e.value)

    with pytest.raises(SyntaxError) as e:
        core = Core(os.path.join(tests_dir, 'parameters_invalidparamtype.core'))
    assert "Invalid paramtype 'badtype' for parameter" in str(e.value)
    
                
def test_get_scripts():
    flag_combos = [{'target' : 'sim'  , 'is_toplevel' : False},
                   {'target' : 'sim'  , 'is_toplevel' : True},
                   {'target' : 'synth', 'is_toplevel' : False},
                   {'target' : 'synth', 'is_toplevel' : True},
    ]
    filename = os.path.join(os.path.dirname(__file__), 'cores', 'misc', 'scriptscore.core')
    core = Core(filename, '', 'dummy_build_root')

    for flags in flag_combos:
        env = {
            'BUILD_ROOT' : 'dummy_build_root',
            'FILES_ROOT' : 'dummyroot'
        }
        result = core.get_scripts("dummyroot", flags)
        expected = {}
        if flags['target'] == 'sim':
            sections = ['post_run', 'pre_build', 'pre_run']
        else:
            if flags['is_toplevel']:
                env['SYSTEM_ROOT'] = core.files_root
                sections = ['pre_build', 'post_build']
            else:
                sections = []
        for section in sections:
            _name = flags['target']+section+'_scripts{}'
            expected[section] = [{'cmd' : ['sh', os.path.join('dummyroot', _name.format(i))],
                                  'name' : _name.format(i),
                                  'env' : env} for i in range(2)]
        assert expected == result

def test_get_tool():
    cores_root = os.path.join(os.path.dirname(__file__), 'cores')

    core = Core(os.path.join(cores_root, 'atlys', 'atlys.core'))
    assert None     == core.get_tool({'target' : 'sim', 'tool' : None})
    assert 'icarus' == core.get_tool({'target' : 'sim', 'tool' : 'icarus'})
    assert 'ise'    == core.get_tool({'target' : 'synth', 'tool' : None})
    assert 'vivado' == core.get_tool({'target' : 'synth', 'tool' : 'vivado'})

    core = Core(os.path.join(cores_root, 'sockit', 'sockit.core'))
    assert 'icarus' == core.get_tool({'target' : 'sim', 'tool' : None})
    assert 'icarus' == core.get_tool({'target' : 'sim', 'tool' : 'icarus'})
    del core.main.backend
    assert None     == core.get_tool({'target' : 'synth', 'tool' : None})
    assert 'vivado' == core.get_tool({'target' : 'synth', 'tool' : 'vivado'})
    core.main.backend = 'quartus'

def test_get_tool_options():
    cores_root = os.path.join(os.path.dirname(__file__), 'cores')

    core = Core(os.path.join(cores_root, 'mor1kx-generic', 'mor1kx-generic.core'))
    assert {'iverilog_options' : ['-DSIM']} == core.get_tool_options({'is_toplevel' : True, 'tool' : 'icarus'})
    assert {} == core.get_tool_options({'is_toplevel' : True, 'tool' : 'modelsim'})
    assert {'fuse_options' : ['some','isim','options']} == core.get_tool_options({'is_toplevel' : True, 'tool' : 'isim'})
    expected = {'xelab_options' : ['--timescale 1ps/1ps', '--debug typical',
                                   'dummy', 'options', 'for', 'xelab']}
    assert expected == core.get_tool_options({'is_toplevel' : True, 'tool' : 'xsim'})
    assert {} == core.get_tool_options({'is_toplevel' : False, 'tool' : 'icarus'})

    core = Core(os.path.join(cores_root, 'elf-loader', 'elf-loader.core'))
    assert {'libs' : ['-lelf']} == core.get_tool_options({'is_toplevel' : False, 'tool' : 'verilator'})
    assert {} == core.get_tool_options({'is_toplevel' : True, 'tool' : 'invalid'})

def test_get_toplevel():
    filename = os.path.join(os.path.dirname(__file__),
                            __name__,
                            "atlys.core")
    core = Core(filename)
    assert 'orpsoc_tb'  == core.get_toplevel({'tool' : 'icarus'})
    assert 'orpsoc_tb'  == core.get_toplevel({'tool' : 'icarus', 'testbench' : None})
    assert 'tb'         == core.get_toplevel({'tool' : 'icarus', 'testbench' : 'tb'})
    assert 'orpsoc_top' == core.get_toplevel({'tool' : 'vivado'})
    filename = os.path.join(os.path.dirname(__file__),
                            __name__,
                            "sockit.core")
    core = Core(filename)
    assert 'dummy_tb'   == core.get_toplevel({'tool' : 'icarus'})
    assert 'dummy_tb'   == core.get_toplevel({'tool' : 'icarus', 'testbench' : None})
    assert 'tb'         == core.get_toplevel({'tool' : 'icarus', 'testbench' : 'tb'})
    assert 'orpsoc_top' == core.get_toplevel({'tool' : 'vivado'})

def test_icestorm():
    filename = os.path.join(os.path.dirname(__file__),
                            __name__,
                            "c3demo.core")
    core = Core(filename)
    assert len(core.file_sets) == 3
    compare_fileset(core.file_sets[0], 'rtl_files', ['c3demo.v', 'ledpanel.v','picorv32.v'])
    compare_fileset(core.file_sets[1], 'tb_files' , ['firmware.hex', '$YOSYS_DAT_DIR/ice40/cells_sim.v', 'testbench.v'])

    #Check that backend files are converted to fileset properly
    compare_fileset(core.file_sets[2], 'backend_files', ['c3demo.pcf'])
    assert core.file_sets[2].file[0].file_type == 'PCF'

    assert core.icestorm.export_files == []
    assert core.icestorm.arachne_pnr_options == ['-s', '1', '-d', '8k']
    assert core.icestorm.top_module == 'c3demo'
    assert core.icestorm.warnings == []
    
def test_ise():
    filename = os.path.join(os.path.dirname(__file__),
                            __name__,
                            "atlys.core")
    core = Core(filename)

    #Check filesets
    assert len(core.file_sets) == 4
    assert core.file_sets[0].name == 'verilog_src_files'
    assert core.file_sets[1].name == 'verilog_tb_src_files'
    assert core.file_sets[2].name == 'verilog_tb_private_src_files'

    #Check that backend files are converted to fileset properly
    compare_fileset(core.file_sets[3], 'backend_files', ['data/atlys.ucf'])
    assert core.file_sets[3].file[0].file_type == 'UCF'

    #Check backend section
    assert core.ise.export_files == []
    assert core.ise.family == 'spartan6'
    assert core.ise.device == 'xc6slx45'
    assert core.ise.package == 'csg324'
    assert core.ise.speed == '-2'
    assert core.ise.top_module == 'orpsoc_top'

    assert core.ise.warnings == []

def test_quartus():
    filename = os.path.join(os.path.dirname(__file__),
                            __name__,
                            "sockit.core")
    core = Core(filename)

    #Check filesets
    assert len(core.file_sets) == 4
    assert core.file_sets[0].name == 'verilog_src_files'
    assert core.file_sets[1].name == 'verilog_tb_src_files'
    assert core.file_sets[2].name == 'verilog_tb_private_src_files'

    #Check that backend files are converted to fileset properly
    assert len(core.file_sets[3].file) == 3
    compare_fileset(core.file_sets[3], 'backend_files', ['data/sockit.qsys', 'data/sockit.sdc', 'data/pinmap.tcl'])
    assert core.file_sets[3].file[0].file_type == 'QSYS'
    assert core.file_sets[3].file[1].file_type == 'SDC'
    assert core.file_sets[3].file[2].file_type == 'tclSource'

    #Check backend section
    assert core.quartus.quartus_options == '--64bit'
    assert core.quartus.family == '"Cyclone V"'
    assert core.quartus.device == '5CSXFC6D6F31C8ES'
    assert core.quartus.top_module == 'orpsoc_top'

    assert core.quartus.warnings == []

def test_simulator():
    #Explicit toplevel
    filename = os.path.join(os.path.dirname(__file__),
                            __name__,
                            "c3demo.core")
    core = Core(filename)
    assert core.simulator['toplevel'] == 'testbench'

    #Implicit toplevel
    filename = os.path.join(os.path.dirname(__file__),
                            __name__,
                            "atlys.core")
    core = Core(filename)
    assert core.simulator['toplevel'] == 'orpsoc_tb'

def test_verilator():
    cores_root = os.path.join(os.path.dirname(__file__), __name__)

    core = Core(os.path.join(cores_root, "verilator_managed_systemc.core"))
    expected = {'cli_parser' : 'managed', 'libs' : [], 'mode' : 'sc'}
    assert expected == core.get_tool_options({'is_toplevel' : True, 'tool' : 'verilator'})

    assert len(core.file_sets) == 2
    compare_fileset(core.file_sets[0], 'verilator_src_files', ['file1.sc', 'file2.sc'])
    assert core.file_sets[0].file[0].file_type == 'systemCSource'
    assert core.file_sets[0].file[1].file_type == 'systemCSource'

    compare_fileset(core.file_sets[1], 'verilator_tb_toplevel', [])



import subprocess
import logging
import sys
import importlib

if sys.version[0] == '2':
    FileNotFoundError = OSError

logger = logging.getLogger(__name__)

class Launcher:
    def __init__(self, cmd, args=[], cwd=None):
        self.cmd      = cmd
        self.args     = args
        self.cwd      = cwd

    def run(self):
        logger.debug(self.cwd)
        logger.debug('    ' + str(self))
        try:
            subprocess.check_call([self.cmd] + self.args,
                                  cwd = self.cwd,
                                  stdin=subprocess.PIPE),
        except FileNotFoundError:
            raise RuntimeError("Command '" + self.cmd + "' not found. Make sure it is in $PATH")
        except subprocess.CalledProcessError:
            self.errormsg = '"{}" exited with an error code. See stderr for details.'
            raise RuntimeError(self.errormsg.format(str(self)))

    def __str__(self):
        return ' '.join([self.cmd] + self.args)

def is_mingw():
    if sys.platform == "msys":
        return True
    return (sys.platform == "win32" and "GCC" in sys.version)

def cygpath(win_path):
    path = subprocess.check_output(["cygpath", "-u", win_path])
    return path.decode('ascii').strip()

import os

def unique_dirs(file_list):
    return list(set([os.path.dirname(f.name) for f in file_list]))


# With help from:
# http://stackoverflow.com/questions/384076/how-can-i-color-python-logging-output
# Very minimal direct copying so should be no license issues.

BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)

# These are the sequences need to get colored output.
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"

COLOR_MAP = {
    'CRITICAL': RED,
    'ERROR': RED,
    'WARNING': YELLOW,
    'INFO': WHITE,
    'DEBUG': WHITE,
    }


class ColoredFormatter(logging.Formatter):

    def __init__(self, msg, monochrome):
        super(ColoredFormatter, self).__init__(msg)
        self.monochrome = monochrome

    def format(self, record):
        uncolored = super(ColoredFormatter, self).format(record)
        levelname = record.levelname
        if not self.monochrome and (levelname in COLOR_MAP):
            color_seq = COLOR_SEQ % (30 + COLOR_MAP[levelname])
            formatted = color_seq + uncolored + RESET_SEQ
        else:
            formatted = uncolored
        return formatted


def setup_logging(level, monchrome=False, log_file=None):
    '''
    Utility function for setting up logging.
    '''
    # Logging to file
    if log_file:
        logging.basicConfig(filename=log_file, filemode='w',
                            level=logging.DEBUG)

    # Pretty color terminal logging
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    formatter = ColoredFormatter("%(levelname)s: %(message)s", monchrome)
    ch.setFormatter(formatter)
    # Which packages do we want to log from.
    packages = ('__main__', 'fusesoc',)
    for package in packages:
        logger = logging.getLogger(package)
        logger.addHandler(ch)
        logger.setLevel(level)
    # Warning only packages
    warning_only_packages = []
    for package in warning_only_packages:
        logger = logging.getLogger(package)
        logger.addHandler(ch)
        logger.setLevel(logging.WARNING)
    logger.debug('Setup logging at level {}.'.format(level))



import os.path

from edalize.edatool import Edatool

class Ise(Edatool):

    tool_options = {'members' : {'family'  : 'String',
                                 'device'  : 'String',
                                 'package' : 'String',
                                 'speed'   : 'String'}}

    argtypes = ['vlogdefine', 'vlogparam']

    MAKEFILE_TEMPLATE = """#Auto generated by Edalize
include config.mk

all: $(TOPLEVEL).bit

$(TOPLEVEL).bit:  $(NAME)_run.tcl $(NAME).xise
	xtclsh $^

$(NAME).xise: $(NAME).tcl
	xtclsh $<
"""

    TCL_RUN_FILE_TEMPLATE = """#Auto generated by Edalize
project open $::argv
process run "Generate Programming File"
"""

    TCL_FILE_TEMPLATE = """#Auto generated by Edalize
project new {design}
project set family {family}
project set device {device}
project set package {package}
project set speed {speed}
project set "Generate Detailed MAP Report" true
"""

    PGM_FILE_TEMPLATE = """
# Batch script for programming the device using a JTAG interface.
# Used with:
# $ impact -batch {pgm_file}

setMode -bscan
setCable -port auto
addDevice -p 1 -file {bit_file}
program -p 1
saveCDF -file {cdf_file}
quit
"""

    def configure_main(self):
        for i in ['family', 'device', 'package', 'speed']:
            if not i in self.tool_options:
                raise RuntimeError("Missing required option '{}'".format(i))
        self._write_tcl_file()
        with open(os.path.join(self.work_root, 'Makefile'),'w') as f:
            f.write(self.MAKEFILE_TEMPLATE)
        with open(os.path.join(self.work_root, 'config.mk'),'w') as f:
            f.write("NAME     := {}\n".format(self.name))
            f.write("TOPLEVEL := {}\n".format(self.toplevel))
        with open(os.path.join(self.work_root, self.name+'_run.tcl'),'w') as f:
            f.write(self.TCL_RUN_FILE_TEMPLATE)

    def _write_tcl_file(self):
        tcl_file = open(os.path.join(self.work_root, self.name+'.tcl'),'w')

        tcl_file.write(self.TCL_FILE_TEMPLATE.format(
            design               = self.name,
            family               = self.tool_options['family'],
            device               = self.tool_options['device'],
            package              = self.tool_options['package'],
            speed                = self.tool_options['speed']))

        if self.vlogdefine:
            s = 'project set "Verilog Macros" "{}" -process "Synthesize - XST"\n'
            tcl_file.write(s.format('|'.join([k+'='+self._param_value_str(v) for k,v in self.vlogdefine.items()])))

        if self.vlogparam:
            s = 'project set "Generics, Parameters" "{}" -process "Synthesize - XST"\n'
            tcl_file.write(s.format('|'.join([k+'='+self._param_value_str(v, '\\"') for k,v in self.vlogparam.items()])))

        (src_files, incdirs) = self._get_fileset_files()

        if incdirs:
            tcl_file.write('project set "Verilog Include Directories" "{}" -process "Synthesize - XST"\n'.format('|'.join(incdirs)))

        _libraries = []
        for f in src_files:
            if f.file_type == 'tclSource':
                tcl_file.write('source {}\n'.format(f.name))
            elif f.file_type.startswith('verilogSource'):
                tcl_file.write('xfile add {}\n'.format(f.name))
            elif f.file_type == 'UCF':
                tcl_file.write('xfile add {}\n'.format(f.name))
            elif f.file_type == 'BMM':
                tcl_file.write('xfile add {}\n'.format(f.name))
            elif f.file_type.startswith('vhdlSource'):
                if f.logical_name:
                    if not f.logical_name in _libraries:
                        tcl_file.write('lib_vhdl new {}\n'.format(f.logical_name))
                        _libraries.append(f.logical_name)
                    _s = 'xfile add {} -lib_vhdl {}\n'
                    tcl_file.write(_s.format(f.name,
                                             f.logical_name))
                else:
                    tcl_file.write('xfile add {}\n'.format(f.name))
            elif f.file_type == 'user':
                pass

        tcl_file.write('project set top "{}"\n'.format(self.toplevel))
        tcl_file.close()

    def run(self, remaining):
        pgm_file_name = os.path.join(self.work_root, self.name+'.pgm')
        self._write_pgm_file(pgm_file_name)
        self._run_tool('impact', ['-batch', pgm_file_name])

    def _write_pgm_file(self, pgm_file_name):
        pgm_file = open(pgm_file_name,'w')
        pgm_file.write(self.PGM_FILE_TEMPLATE.format(
            pgm_file             = pgm_file_name,
            bit_file             = os.path.join(self.work_root, self.toplevel+'.bit'),
            cdf_file             = os.path.join(self.work_root, self.toplevel+'.cdf')))
        pgm_file.close()



import pytest

def test_modelsim():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['plusarg', 'vlogdefine', 'vlogparam']
    name         = 'test_modelsim_0'
    tool         = 'modelsim'
    tool_options = {
        'vlog_options' : ['some', 'vlog_options'],
        'vsim_options' : ['a', 'few', 'vsim_options'],
    }

    #FIXME: Add VPI tests
    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options, use_vpi=False)
    backend.configure(args)

    compare_files(ref_dir, work_root, [
        'Makefile',
        'edalize_build_rtl.tcl',
        'edalize_main.tcl',
    ])

    orig_env = os.environ.copy()
    os.environ['MODEL_TECH'] = os.path.join(tests_dir, 'mock_commands')

    backend.build()
    os.makedirs(os.path.join(work_root, 'work'))

    compare_files(ref_dir, work_root, ['vsim.cmd'])

    backend.run(args)

    with open(os.path.join(ref_dir, 'vsim2.cmd')) as fref, open(os.path.join(work_root, 'vsim.cmd')) as fgen:
        assert fref.read() == fgen.read()

    os.environ = orig_env



import logging
import os

from okonomiyaki.versions import EnpkgVersion

from simplesat.constraints import PrettyPackageStringParser, Requirement
from simplesat.dependency_solver import DependencySolver
from simplesat.errors import NoPackageFound, SatisfiabilityError
from simplesat.pool import Pool
from simplesat.repository import Repository
from simplesat.request import Request

from fusesoc.core import Core

logger = logging.getLogger(__name__)

class DependencyError(Exception):
    def __init__(self, value, msg=""):
        self.value = value
        self.msg = msg
    def __str__(self):
        return repr(self.value)

class CoreDB(object):
    def __init__(self):
        self._cores = {}

    #simplesat doesn't allow ':', '-' or leading '_'
    def _package_name(self, vlnv):
        _name = "{}_{}_{}".format(vlnv.vendor,
                                  vlnv.library,
                                  vlnv.name).lstrip("_")
        return _name.replace('-','__')

    def _package_version(self, vlnv):
        return "{}-{}".format(vlnv.version,
                              vlnv.revision)

    def _parse_depend(self, depends):
        #FIXME: Handle conflicts
        deps = []
        _s = "{} {} {}"
        for d in depends:
            deps.append(_s.format(self._package_name(d),
                                  d.relation,
                                  self._package_version(d)))
        return ", ".join(deps)

    def add(self, core):
        name = str(core.name)
        logger.debug("Adding core " + name)
        if name in self._cores:
            _s = "Replacing {} in {} with the version found in {}"
            logger.debug(_s.format(name,
                                   self._cores[name].core_root,
                                   core.core_root))
        self._cores[name] = core

    def find(self, vlnv=None):
        if vlnv:
            found = self._solve(vlnv, only_matching_vlnv=True)[-1]
        else:
            found = list(self._cores.values())
        return found

    def solve(self, top_core, flags):
        return self._solve(top_core, flags)

    def _solve(self, top_core, flags={}, only_matching_vlnv=False):
        def eq_vln(this, that):
            return \
                this.vendor  == that.vendor and \
                this.library == that.library and \
                this.name    == that.name

        repo = Repository()
        _flags = flags.copy()
        for core in self._cores.values():
            if only_matching_vlnv:
                if not eq_vln(core.name, top_core):
                    continue

            package_str = "{} {}-{}".format(self._package_name(core.name),
                                            core.name.version,
                                            core.name.revision)
            if not only_matching_vlnv:
                _flags['is_toplevel'] = (core.name == top_core)
                _depends = core.get_depends(_flags)
                if _depends:
                    _s = "; depends ( {} )"
                    package_str += _s.format(self._parse_depend(_depends))
            parser = PrettyPackageStringParser(EnpkgVersion.from_string)

            package = parser.parse_to_package(package_str)
            package.core = core

            repo.add_package(package)

        request = Request()
        _top_dep = "{} {} {}".format(self._package_name(top_core),
                                     top_core.relation,
                                     self._package_version(top_core))
        requirement = Requirement._from_string(_top_dep)
        request.install(requirement)
        installed_repository = Repository()
        pool = Pool([repo])
        pool.add_repository(installed_repository)
        solver = DependencySolver(pool, [repo], installed_repository)

        try:
            transaction = solver.solve(request)
        except SatisfiabilityError as e:
            raise DependencyError(top_core.name,
                                  msg=e.unsat.to_string(pool))
        except NoPackageFound as e:
            raise DependencyError(top_core.name)

        return [op.package.core for op in transaction.operations]

class CoreManager(object):

    def __init__(self, config):
        self.config = config
        self._cores_root = []
        self.db = CoreDB()

    def load_cores(self, path):
        if os.path.isdir(path) == False:
            raise IOError(path + " is not a directory")
        logger.debug("Checking for cores in " + path)
        for root, dirs, files in os.walk(path, followlinks=True):
            if 'FUSESOC_IGNORE' in files:
                del dirs[:]
                continue
            for f in files:
                if f.endswith('.core'):
                    core_file = os.path.join(root, f)
                    try:
                        core = Core(core_file, self.config.cache_root, self.config.build_root)
                        self.db.add(core)
                    except SyntaxError as e:
                        w = "Parse error. Ignoring file " + core_file + ": " + e.msg
                        logger.warning(w)
                    except ImportError as e:
                        w = 'Failed to register "{}" due to unknown provider: {}'
                        logger.warning(w.format(core_file, str(e)))

    def add_cores_root(self, path):
        if not path:
            return

        abspath = os.path.abspath(os.path.expanduser(path))
        if abspath in self._cores_root:
            return

        self.load_cores(os.path.expanduser(path))
        self._cores_root += [abspath]

    def get_cores_root(self):
        return self._cores_root

    def get_depends(self, core, flags):
        logger.debug("Calculating dependencies for {}{} with flags {}".format(core.relation,str(core), str(flags)))
        resolved_core = self.db.find(core)
        deps = self.db.solve(resolved_core.name, flags)
        logger.debug(" Resolved core to {}".format(str(resolved_core.name)))
        logger.debug(" with dependencies " + ', '.join([str(c.name) for c in deps]))
        return deps

    def get_cores(self):
        return {str(x.name) : x for x in self.db.find()}

    def get_core(self, name):
        c = self.db.find(name)
        c.name.relation = "=="
        return c



import os
import logging
from fusesoc.utils import unique_dirs
from fusesoc.vlnv import Vlnv

logger = logging.getLogger(__name__)

class File(object):
    """File objects consist of a mandatory file name, with path relative to
the core root. Extra options can be specified as a comma-separated list
enclosed in [] after the file name. Options are either boolean (option) or has a 
value (option=value). No white-space is allowed anywhere in the file object

The following options are defined:

* *file_type :* Value can be any type defined in <<FileTypes, File types>>

* *is_include_file :* Boolean value to indicate this should be treated as an include file

* *logical_name :* Indicate that the file belongs to a logical unit (e.g. VHDL Library) with the name set by the value
* *copyto :* Indicate that the file should be copied to a new location relative to the work root.

Example: rtl/verilog/uart_defines.v[file_type=verilogSource,is_include_file]

Example: data/mem_init_file.bin[copyto=out/boot.bin]

"""
    FILE_TYPES = [
        'PCF',
        'QIP',
        'SDC',
        'UCF',
        'BMM',
        'tclSource',
        'user',
        'verilogSource',
        'verilogSource-95',
        'verilogSource-2001',
        'verilogSource-2005',
        'systemVerilogSource',
        'systemVerilogSource-3.0',
        'systemVerilogSource-3.1',
        'systemVerilogSource-3.1a',
        'vhdlSource',
        'vhdlSource-87',
        'vhdlSource-93',
        'vhdlSource-2008',
        'xci',
        'xdc',
        ]
    name      = ""
    copyto    = ""
    file_type = ""
    is_include_file = False
    logical_name = ""
    def __init__(self, s):
        self.is_include_file = False
        if s[-1:] == ']':
            _tmp = s[:-1].split('[')
            if(len(_tmp) != 2):
                raise SyntaxError("Expected '['")
            self.name = _tmp[0]
            for _arg in [x.strip() for x in _tmp[1].split(',')]:
                if _arg == "is_include_file":
                    self.is_include_file = True
                elif '=' in _arg:
                    _tmp = [x.strip() for x in _arg.split('=')]
                    if _tmp[0] == 'file_type' and _tmp[1] not in self.FILE_TYPES:
                        _s = "Unknown file type '{}'. Allowed file types are {}"
                        raise SyntaxError(_s.format(_tmp[1], ', '.join(self.FILE_TYPES)))
                    if _tmp[0] in ['copyto', 'file_type', 'logical_name']:
                        setattr(self, _tmp[0], _tmp[1])
                else:
                    raise SyntaxError("Unexpected argument '"+_arg+"'")
        else:
            self.name = s

class Error(Exception):
    pass


class NoSuchItemError(Error):
    pass


class UnknownSection(Error):
    pass

class StringList(list):
    """Space-separated list of strings"""
    def __new__(cls, *args, **kwargs):
        if not args:
            return list()
        else:
            return list(args[0].split())

class PathList(StringList):
    """Space-separated list of paths

Each element in the list is subjected to expansion of environment variables and
 ~ to home directories
"""
    def __new__(cls, *args, **kwargs):
        if not args:
            return list()
        else:
            return [os.path.expandvars(p) for p in args[0].split()]

class FileList(PathList):
    """Space-separated list of <<File>>

Each element in the list is first subjected to the expansion according to
<<PathList>> and then parsed as a <<File>>
"""
    def __new__(clk, *args, **kwargs):
        if not args:
            return list()
        else:
            return [File(p) for p in PathList(args[0])]

class VlnvList(StringList):
    """Space-separated list of VLNV tags

Each element is treated as a VLNV element with an optional version range

Example: librecores.org:peripherals:uart16550:1.5 >=::simple_spi:1.6 mor1kx =::i2c:1.14
"""
    def __new__(clk, *args, **kwargs):
        if not args:
            return list()
        else:
            return [Vlnv(p) for p in StringList(args[0])]

class EnumList(list):
    def __new__(cls, *args, **kwargs):
        if not args:
            return super(EnumList, cls).__new__(cls)
        else:
            values = kwargs['values']
            _args = args[0].split()
            _valid = []
            _invalid = []
            for arg in _args:
                if arg in values:
                    _valid.append(arg)
                else:
                    _invalid.append(arg)
            if _invalid:
                raise ValueError(' '.join(_valid), _invalid, values)
            return _valid

class SimulatorList(EnumList):
    """List of supported simulators. Allowed values are ghdl, icarus, isim, modelsim, verilator, xsim"""
    def __new__(cls, *args, **kwargs):
        values = ['ghdl', 'icarus', 'modelsim', 'verilator', 'isim', 'xsim']
        return super(SimulatorList, cls).__new__(cls, *args, values=values)

class SourceType(str):
    """Language used for Verilator testbenches. Allowed values are C, CPP or systemC"""
    def __new__(cls, *args, **kwargs):
        if args:
            arg = args[0]
            values = ['C', 'CPP', 'systemC']
            if arg in values:
                return str(arg)
            raise ValueError("Invalid value '" + str(arg) + "'. Allowed values are '" + "', '".join(values)+"'")
        return str

class Section(object):

    TAG = None
    named = False
    def __init__(self):
        self._members = {}
        self.export_files = []
        self.warnings = []

    def _add_member(self, name, _type, desc):
        if name in self._members:
            _s = "{}: the section '{}' is already defined"
            raise ValueError(_s.format(self.__class__.__name__, name))
        self._members[name] = {'type' : _type, 'desc' : desc}
        setattr(self, name, _type())

    def export(self):
        return self.export_files

    def load_dict(self, items):
        for item in items:
            if item in self._members:
                _type = self._members.get(item)['type']
                try:
                    setattr(self, item, _type(items.get(item)))
                except ValueError as e:
                    _s = "Invalid value '{}'. Allowed values are '{}'"
                    logger.warning(_s.format(', '.join(e.args[1]),
                                      ', '.join(e.args[2])))
                    setattr(self, item, _type(e.args[0]))
            else:
                self.warnings.append(
                        'Unknown item "%(item)s" in section "%(section)s"' % {
                            'item': item, 'section': self.TAG})

    def __str__(self):
        s = ''
        for k,v in self._members.items():
            if isinstance(v.get('type'), list):
                s += k + ' : ' + ';'.join(getattr(self, item)) + '\n'
            elif isinstance(v.get('type'), str):
                s += k + ' : ' + getattr(self, k) + '\n'
        return s

class ScriptsSection(Section):
    TAG = 'scripts'
    def __init__(self, items=None):
        super(ScriptsSection, self).__init__()
        self._add_member('pre_synth_scripts', StringList, 'Scripts to run before backend synthesis')
        self._add_member('post_impl_scripts', StringList, 'Scripts to run after backend implementation')
        self._add_member('pre_build_scripts', StringList, 'Scripts to run before building')
        self._add_member('pre_run_scripts'  , StringList, 'Scripts to run before running simulations')
        self._add_member('post_run_scripts' , StringList, 'Scripts to run after simulations')

        if items:
            self.load_dict(items)

class ToolSection(Section):
    def __init__(self):
        super(ToolSection, self).__init__()
        self._add_member('depend', VlnvList, "Tool-specific Dependencies")
    def __str__(self):
        s = ""
        if self.depend:
            _s = "{}-specific dependencies : {}\n"
            s += _s.format(self.TAG,
                     ' '.join([x.depstr() for x in self.depend]))
        return(s)

class MainSection(Section):
    TAG = 'main'

    def __init__(self, items=None):
        super(MainSection, self).__init__()

        self._add_member('name'       , str     , "Component name")
        self._add_member('backend'    , str     , "Backend for FPGA implementation")
        self._add_member('component'  , PathList, "Core IP-Xact component file")
        self._add_member('description', str, "Core description")
        self._add_member('depend'     , VlnvList, "Common dependencies")
        self._add_member('simulators' , SimulatorList, "Supported simulators. Valid values are icarus, modelsim, verilator, isim and xsim. Each simulator have a dedicated section desribed elsewhere in this document")
        self._add_member('patches'    , StringList, "FuseSoC-specific patches")

        if items:
            self.load_dict(items)

class VhdlSection(Section):

    TAG = 'vhdl'

    def __init__(self, items=None):
        super(VhdlSection, self).__init__()

        self._add_member('src_files', PathList, "VHDL source files for simulation and synthesis")

        if items:
            self.load_dict(items)
            self.export_files = self.src_files

class VerilogSection(Section):

    TAG = 'verilog'

    def __init__(self, items=None):
        super(VerilogSection, self).__init__()

        self.include_dirs = []
        self.tb_include_dirs = []

        self._add_member('src_files'           , FileList, "Verilog source files for synthesis/simulation")
        self._add_member('include_files'       , FileList, "Verilog include files")
        self._add_member('tb_src_files'        , FileList, "Verilog source files that are only used in simulation. Visible to other cores")
        self._add_member('tb_private_src_files', FileList, "Verilog source files that are only used in the core's own testbench. Not visible to other cores")
        self._add_member('tb_include_files'    , FileList, "Testbench include files")
        self._add_member('file_type'           , str     , "Default file type of the files in fileset")

        if items:
            self.load_dict(items)
            if not self.file_type:
                self.file_type = "verilogSource"
            if self.include_files:
                self.include_dirs  += unique_dirs(self.include_files)
            if self.tb_include_files:
                self.tb_include_dirs  += unique_dirs(self.tb_include_files)

            self.export_files = self.src_files + self.include_files + self.tb_src_files + self.tb_include_files + self.tb_private_src_files

class FileSetSection(Section):
    TAG = 'fileset'
    named = True
    def __init__(self, items=None):
        super(FileSetSection, self).__init__()

        self._add_member('files'          , FileList, "List of files in fileset")
        self._add_member('file_type'      , str     , "Default file type of the files in fileset")
        self._add_member('is_include_file', str     , "Specify all files in fileset as include files")
        self._add_member('logical_name'   , str     , "Default logical_name (e.g. library) of the files in fileset")
        self._add_member('scope'          , str     , "Visibility of fileset (private/public). Private filesets are only visible when this core is the top-level. Public filesets are visible also for cores that depend on this core. Default is public")
        self._add_member('usage'          , StringList, "List of tags describing when this fileset should be used. Can be general such as sim or synth, or tool-specific such as quartus, verilator, icarus. Defaults to 'sim synth'.")
        if items:
            self.load_dict(items)
            if not self.scope:
                self.scope = 'public'
            if not self.usage:
                self.usage = ['sim', 'synth']
            for f in self.files:
                if not f.file_type:
                    f.file_type = self.file_type
                if self.is_include_file.lower() == "true":
                    f.is_include_file = True
                if not f.logical_name:
                    f.logical_name = self.logical_name
            self.export_files = self.files


class VpiSection(Section):

    TAG = 'vpi'

    def __init__(self, items=None):
        super(VpiSection, self).__init__()

        self.include_dirs = []

        self._add_member('src_files'    , FileList, "C source files for VPI library")
        self._add_member('include_files', FileList, "C include files for VPI library")
        self._add_member('libs'         , StringList, "External libraries linked with the VPI library")

        if items:
            self.load_dict(items)
            if self.include_files:
                self.include_dirs  += unique_dirs(self.include_files)

            self.export_files = self.src_files + self.include_files


class ModelsimSection(ToolSection):

    TAG = 'modelsim'

    def __init__(self, items=None):
        super(ModelsimSection, self).__init__()

        self._add_member('vlog_options', StringList, "Additional arguments for vlog")
        self._add_member('vsim_options', StringList, "Additional arguments for vsim")

        if items:
            self.load_dict(items)

class RivieraproSection(ToolSection):

    TAG = 'rivierapro'

    def __init__(self, items=None):
        super(RivieraproSection, self).__init__()

        self._add_member('vlog_options', StringList, "Additional arguments for vlog")
        self._add_member('vsim_options', StringList, "Additional arguments for vsim")

        if items:
            self.load_dict(items)

class GhdlSection(ToolSection):
    TAG = 'ghdl'

    def __init__(self, items=None):
        super(GhdlSection, self).__init__()

        self._add_member('analyze_options', StringList, "Extra GHDL analyzer options")
        self._add_member('run_options', StringList, "Extra GHDL run options")

        if items:
            self.load_dict(items)

    def __str__(self):
        s = super(GhdlSection, self).__str__()
        if self.analyze_options: s += "Extra GHDL analyzer options : {}\n".format(' '.join(self.analyze_options))
        if self.run_options: s += "Extra GHDL run options : {}\n".format(' '.join(self.run_options))
        return s

class IcarusSection(ToolSection):

    TAG = 'icarus'

    def __init__(self, items=None):
        super(IcarusSection, self).__init__()

        self._add_member('iverilog_options', StringList, "Extra Icarus verilog compile options")

        if items:
            self.load_dict(items)

    def __str__(self):
        s = super(IcarusSection, self).__str__()
        if self.iverilog_options: s += "Icarus compile options : {}\n".format(' '.join(self.iverilog_options))
        return s


class IsimSection(ToolSection):

    TAG = 'isim'

    def __init__(self, items=None):
        super(IsimSection, self).__init__()

        self._add_member('isim_options', StringList, "Extra Isim compile options")

        if items:
            self.load_dict(items)

    def __str__(self):
        s = super(IsimSection, self).__str__()
        if self.isim_options: s += "Isim compile options : {}\n".format(' '.join(self.isim_options))
        return s


class XsimSection(ToolSection):

    TAG = 'xsim'

    def __init__(self, items=None):
        super(XsimSection, self).__init__()

        self._add_member('xsim_options', StringList, "Extra Xsim compile options")

        if items:
            self.load_dict(items)

    def __str__(self):
        s = super(XsimSection, self).__str__()
        if self.xsim_options: s += "Xsim compile options : {}\n".format(' '.join(self.xsim_options))
        return s

class VerilatorSection(ToolSection):

    TAG = 'verilator'

    def __init__(self, items=None):
        super(VerilatorSection, self).__init__()

        self.include_dirs = []

        self._add_member('verilator_options', StringList, "Verilator build options")
        self._add_member('src_files'        , FileList  , "Verilator testbench C/cpp/sysC source files")
        self._add_member('include_files'    , FileList  , "Verilator testbench C include files")
        self._add_member('define_files'     , PathList  , "Verilog include files containing `define directives to be converted to C #define directives in corresponding .h files (deprecated)")
        self._add_member('libs'             , PathList  , "External libraries linked with the generated model")

        self._add_member('tb_toplevel', FileList, 'Testbench top-level C/C++/SC file')
        self._add_member('source_type', str, 'Testbench source code language (Legal values are systemC, C, CPP. Default is C)')
        self._add_member('top_module' , str, 'verilog top-level module')
        self._add_member('cli_parser' , str, "Select CLI argument parser. Set to 'fusesoc' to handle parameter sections like other simulators. Set to 'passthrough' to send the arguments directly to the verilated model. Default is 'passthrough'")

        if items:
            self.load_dict(items)
            self.include_dirs  = unique_dirs(self.include_files)
            if not self.source_type:
                self.source_type = 'C'

    def __str__(self):
        s = super(VerilatorSection, self).__str__()
        s += """Verilator options       : {verilator_options}
External libraries      : {libs}
Verilog top module      : {top_module}
"""
        return s.format(verilator_options=' '.join(self.verilator_options),
                        libs=' '.join(self.libs),
                        top_module=self.top_module)

class IcestormSection(ToolSection):

    TAG = 'icestorm'

    def __init__(self, items=None):
        super(IcestormSection, self).__init__()

        self._add_member('arachne_pnr_options', StringList, "arachne-pnr options")
        self._add_member('yosys_synth_options' , StringList, "Additional options for the synth_* commands in yosys")
        self._add_member('pcf_file' , FileList, "Physical constraint file")
        self._add_member('top_module', str, 'RTL top-level module')

        if items:
            self.load_dict(items)

class VivadoSection(ToolSection):

    TAG = 'vivado'

    def __init__(self, items=None):
        super(VivadoSection, self).__init__()

        self._add_member('part'       , str, 'FPGA device part')
        self._add_member('hw_device'  , str, 'FPGA device identifier')
        self._add_member('top_module' , str, 'RTL top-level module')

        if items:
            self.load_dict(items)

class IseSection(ToolSection):

    TAG = 'ise'

    def __init__(self, items=None):
        super(IseSection, self).__init__()

        self._add_member('ucf_files' , FileList, "UCF constraint files")
        self._add_member('tcl_files' , FileList, "Extra TCL scripts")
        self._add_member('family'    , str, 'FPGA device family')
        self._add_member('device'    , str, 'FPGA device identifier')
        self._add_member('package'   , str, 'FPGA device package')
        self._add_member('speed'     , str, 'FPGA device speed grade')
        self._add_member('top_module', str, 'RTL top-level module')

        if items:
            self.load_dict(items)

class QuartusSection(ToolSection):

    TAG = 'quartus'

    def __init__(self, items=None):
        super(QuartusSection, self).__init__()

        self._add_member('qsys_files', FileList, "Qsys IP description files")
        self._add_member('sdc_files' , FileList, "SDC constraint files")
        self._add_member('tcl_files' , FileList, "Extra script files")

        self._add_member('quartus_options', str, 'Quartus command-line options')
        self._add_member('family'         , str, 'FPGA device family')
        self._add_member('device'         , str, 'FPGA device identifier')
        self._add_member('top_module'     , str, 'RTL top-level module')

        self.top_module = 'orpsoc_top'
        if items:
            self.load_dict(items)

    def __str__(self):
        s = ''
        for x in ['family', 'device', 'top_module']:
            s += "{} : {}\n".format(self._members[x]['desc'], getattr(self, x))
        return s

class ParameterSection(Section):
    TAG = 'parameter'
    named = True
    def __init__(self, items=None):
        super(ParameterSection, self).__init__()

        self._add_member('datatype'   , str, 'Data type of argument (int, str, bool, file')
        self._add_member('default'    , str, 'Default value of argument')
        self._add_member('description', str, 'Parameter description')
        self._add_member('paramtype'  , str, 'Type of parameter (plusarg, vlogparam, generic, cmdlinearg')
        self._add_member('scope'      , str, 'Visibility of parameter. Private parameters are only visible when this core is the top-level. Public parameters are visible also when this core is pulled in as a dependency of another core')

        if items:
            self.load_dict(items)
            if not self.datatype in ['bool', 'file', 'int', 'str']:
                _s = "Invalid datatype '{}' for parameter"
                raise SyntaxError(_s.format(self.datatype))
            if not self.paramtype in ['cmdlinearg', 'generic', 'plusarg',
                                      'vlogdefine', 'vlogparam']:
                _s = "Invalid paramtype '{}' for parameter"
                raise SyntaxError(_s.format(self.paramtype))
            if self.default:
                if self.datatype == 'bool':
                    if self.default == 'true':
                        self.default = True
                    elif self.default == 'false':
                        self.default = False
                    else:
                        _s = "Invalid default value '{}' for bool parameter"
                        raise SyntaxError(_s.format(self.default))
                elif self.datatype == 'int':
                    self.default = int(self.default)
                
    def __str__(self):
        return """Data type      : {}
Default value  : {}
Description    : {}
Parameter type : {}
Scope          : {}
""".format(self.datatype, self.default, self.description, self.paramtype, self.scope)

def load_section(config, section_name, file_name='<unknown>'):
    tmp = section_name.split(' ')
    _type = tmp[0]
    if len(tmp) == 2:
        _name = tmp[1]
    else:
        _name = None
    cls = SECTION_MAP.get(_type)
    if cls is None:
        #Note: The following sections are not in section.py yet
        if not section_name in ['plusargs', 'simulator', 'provider']:
            logger.warning("Unknown section '{}' in '{}'".format(section_name, file_name))
        return None

    items = config.get_section(section_name)
    section = cls(items)
    if section.warnings:
        for warning in section.warnings:
            logger.warning('%s in %s' % (warning, file_name))
    if _name:
        return (section, _name)
    else:
        return section


def load_all(config, file_name='<unknown>'):
    for section_name in config.sections():
        section = load_section(config, section_name, file_name)
        if section:
            yield section


SECTION_MAP = {}


def _register_subclasses(parent):
    for cls in parent.__subclasses__():
        _register_subclasses(cls)
        if cls.TAG is None:
            continue
        SECTION_MAP[cls.TAG] = cls


_register_subclasses(Section)

if __name__ == "__main__":
    FILE_TEMPLATE = """CAPI1 Definition
===============
:toc:

Type definitions
----------------
{types}

[[FileTypes]]
File types
----------

The following valid file types are defined: {filetypes}

Sections
--------
{sections}
"""

    TYPE_TEMPLATE = """

[[{name}]]
{name}
{bar}
{doc}

"""
    SECTION_TEMPLATE = """

{}
{}

[cols="2,1,5",options="header"]
|==============================
|Name | Type | Description
{}
|==============================

"""

    types = ""
    for _type in [File, FileList, PathList, SimulatorList, SourceType, StringList, VlnvList]:
        types += TYPE_TEMPLATE.format(name = _type.__name__,
                                      bar  = '~'*len(_type.__name__),
                                      doc  = _type.__doc__)
    filetypes = ', '.join(File.FILE_TYPES)

    sections = ""
    for k,v in sorted(SECTION_MAP.items()):
        s = []
        for k2, v2 in sorted(v()._members.items()):
            _typename = v2['type'].__name__
            if _typename == 'str':
                _typename = "String"
            else:
                _typename = "<<{},{}>>".format(_typename, _typename)
            s.append("|{} | {} | {}".format(k2,
                                            _typename,
                                            v2['desc']))
        sections += SECTION_TEMPLATE.format(k, '~'*len(k), '\n'.join(s))

    print(FILE_TEMPLATE.format(types=types,
                               filetypes = filetypes,
                               sections = sections))



import logging
import os.path
import sys
import tarfile
import zipfile
import shutil

logger = logging.getLogger(__name__)

if sys.version_info[0] >= 3:
    import urllib.request as urllib
    from urllib.error import URLError
    from urllib.error import HTTPError
else:
    import urllib
    from urllib2 import URLError
    from urllib2 import HTTPError

from fusesoc.provider.provider import Provider

class Url(Provider):

    def _checkout(self, local_dir):
        url = self.config.get('url')
        logger.info("Downloading...")
        user_agent = self.config.get('user-agent')
        if user_agent and sys.version_info[0] >= 3:
            opener = urllib.build_opener()
            opener.addheaders = [('User-agent', user_agent)]
            urllib.install_opener(opener)
        try:
            (filename, headers) = urllib.urlretrieve(url)
        except (URLError, HTTPError) as e:
            raise RuntimeError("Failed to download '{}'. '{}'".format(url, e.reason))

        filetype = self.config.get('filetype')
        if filetype == 'tar':
            t = tarfile.open(filename)
            t.extractall(local_dir)
        elif filetype == 'zip':
            with zipfile.ZipFile(filename, "r") as z:
                z.extractall(local_dir)
        elif filetype == 'simple':
            _filename = url.rsplit('/', 1)[1]
            os.makedirs(local_dir)
            shutil.copy2(filename, os.path.join(local_dir, _filename))
        else:
            raise RuntimeError("Unknown file type '" + filetype + "' in [provider] section")



from collections import OrderedDict
import logging

import sys
if sys.version[0] == '2':
    import ConfigParser as configparser
else:
    import configparser

import os
import importlib

logger = logging.getLogger(__name__)

class Config(object):

    def __init__(self, path=None, file=None):
        self.build_root = None
        self.cache_root = None
        self.cores_root = []
        self.systems_root = None
        self.library_root = None
        self.libraries = OrderedDict()

        config = configparser.SafeConfigParser()
        if file is None:
            if path is None:
                xdg_config_home = os.environ.get('XDG_CONFIG_HOME') or \
                          os.path.join(os.path.expanduser('~'), '.config')
                config_files = ['/etc/fusesoc/fusesoc.conf',
                        os.path.join(xdg_config_home, 'fusesoc','fusesoc.conf'),
                        'fusesoc.conf']
            else:
                logger.debug("Using config file '{}'".format(path))
                if not os.path.isfile(path):
                    with open(path, 'a'):
                        pass
                config_files = [path]

            logger.debug('Looking for config files from ' + ':'.join(config_files))
            files_read = config.read(config_files)
            logger.debug('Found config files in ' + ':'.join(files_read))
            if files_read:
                self._path = files_read[-1]
        else:
            logger.debug('Using supplied config file')
            if sys.version[0] == '2':
                config.readfp(file)
            else:
                config.read_file(file)
            file.seek(0)
            self._path = file.name

        for item in ['build_root', 'cache_root', 'systems_root', 'library_root']:
            try:
                setattr(self, item, os.path.expanduser(config.get('main', item)))
            except configparser.NoOptionError:
                pass
            except configparser.NoSectionError:
                pass
        item = 'cores_root'
        try:
            setattr(self, item, config.get('main', item).split())
        except configparser.NoOptionError:
            pass
        except configparser.NoSectionError:
            pass

        #Set fallback values
        if self.build_root is None:
            self.build_root   = os.path.abspath('build')
        if self.cache_root is None:
            xdg_cache_home = os.environ.get('XDG_CACHE_HOME') or \
                             os.path.join(os.path.expanduser('~'), '.cache')
            self.cache_root = os.path.join(xdg_cache_home, 'fusesoc')
            if not os.path.exists(self.cache_root):
                os.makedirs(self.cache_root)
        if not self.cores_root and os.path.exists('cores'):
            self.cores_root   = [os.path.abspath('cores')]
        if self.systems_root is None and os.path.exists('systems'):
            self.systems_root = os.path.abspath('systems')
        if self.library_root is None:
            xdg_data_home = os.environ.get('XDG_DATA_HOME') or \
                             os.path.join(os.path.expanduser('~'), '.local/share')
            self.library_root = os.path.join(xdg_data_home, 'fusesoc')

        # Parse library sections
        library_sections = [x for x in config.sections() if x.startswith('library')]
        for section in library_sections:
            library = section.partition('.')[2]
            try:
                location = config.get(section, 'location')
            except configparser.NoOptionError:
                location = os.path.join(self.library_root, library)

            try:
                auto_sync = config.getboolean(section, 'auto-sync')
            except configparser.NoOptionError:
                auto_sync = True
            except ValueError as e:
                _s = "Error parsing auto-sync '{}'. Ignoring library '{}'"
                logger.warn(_s.format(str(e), library))
                continue

            try:
                sync_uri = config.get(section, 'sync-uri')
            except configparser.NoOptionError:
                # sync-uri is absent for local libraries
                sync_uri = None

            try:
                sync_type = config.get(section, 'sync-type')
            except configparser.NoOptionError:
                # sync-uri is absent for local libraries
                sync_type = None

            self.libraries[library] = {
                    'location': location,
                    'auto-sync': auto_sync,
                    'sync-uri': sync_uri,
                    'sync-type': sync_type
                }

        logger.debug('build_root='+self.build_root)
        logger.debug('cache_root='+self.cache_root)
        logger.debug('cores_root='+':'.join(self.cores_root))
        logger.debug('systems_root='+self.systems_root if self.systems_root else "Not defined")
        logger.debug('library_root='+self.library_root)

    def add_library(self, name, library):
        from fusesoc.provider import get_provider
        if not hasattr(self, '_path'):
            raise RuntimeError("No FuseSoC config file found - can't add library")
        section_name = 'library.' + name

        config = configparser.SafeConfigParser()
        config.read(self._path)

        if not section_name in config.sections():
            config.add_section(section_name)

        # This is not user-controlled at all so an assert is OK
        assert('location' in library or 'sync-uri' in library);

        # sync-uri is absent for local libraries
        if 'sync-uri' in library:
            config.set(section_name, 'sync-uri', library['sync-uri'])

        if 'sync-type' in library:
            config.set(section_name, 'sync-type', library['sync-type'])
        else:
            library['sync-type'] = 'git'

        if 'auto-sync' in library:
            if library['auto-sync']:
                config.set(section_name, 'auto-sync', 'true')
            else:
                config.set(section_name, 'auto-sync', 'false')
        else:
            library['auto-sync'] = True

        if 'location' in library:
            config.set(section_name, 'location', library['location'])
        else:
            library['location'] = os.path.join(self.library_root, name)

        self.libraries[name] = library

        try:
            provider = get_provider(library['sync-type'])
        except ImportError as e:
            raise RuntimeError("Invalid sync-type '{}'".format(library['sync-type']))

        provider.init_library(library)

        with open(self._path, 'w') as conf_file:
            config.write(conf_file)



import pytest

def test_ghdl():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['vlogparam']
    name         = 'test_ghdl'
    tool         = 'ghdl'
    tool_options = {'analyze_options' : ['some', 'analyze_options'],
                    'run_options'     : ['a', 'few', 'run_options']}

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options)
    backend.configure(args)

    compare_files(ref_dir, work_root, ['Makefile'])

    backend.build()
    compare_files(ref_dir, work_root, ['analyze.cmd'])

    backend.run(args)
    compare_files(ref_dir, work_root, ['elab-run.cmd'])



import logging
import os.path
from edalize.edatool import Edatool

logger = logging.getLogger(__name__)

class Ghdl(Edatool):

    tool_options = {'lists' : {'analyze_options' : 'String',
                               'run_options'     : 'String'}}
    argtypes = ['vlogparam']

    def configure_main(self):
        (src_files, incdirs) = self._get_fileset_files()
        # ghdl does not support mixing incompatible versions
        # specifying 93c as std should allow 87 syntax
        # 2008 can't be combined so try to parse everthing with 08 std
        has87 = has93 = has08 = False
        for f in src_files:
            if f.file_type == "vhdlSource-87":
                has87 = True
            elif f.file_type == "vhdlSource-93":
                has93 = True
            elif f.file_type == "vhdlSource-2008":
                has08 = True
        stdarg = []
        if has08:
            if has87 or has93:
                logger.warning("ghdl can't mix vhdlSource-2008 with other standard version\n"+
                               "Trying with treating all as vhdlSource-2008"
                )
            stdarg = ['--std=08']
        elif has87 and has93:
            stdarg = ['--std=93c']
        elif has87:
            stdarg = ['--std=87']
        elif has93:
            stdarg = ['--std=93']

        analyze_options = self.tool_options.get('analyze_options', '')

        run_options = self.tool_options.get('run_options', [])

        makefile = open(os.path.join(self.work_root, 'Makefile'), 'w')
        makefile.write("""#Auto generated by Edalize
STD = {std}
TOPLEVEL = {toplevel}
ANALYZE_OPTIONS = {analyze_options}
RUN_OPTIONS = {run_options}

all: analyze

run:
	ghdl --elab-run $(STD) $(TOPLEVEL) $(RUN_OPTIONS) $(EXTRA_OPTIONS)
analyze:
""".format(std=' '.join(stdarg),
           toplevel=self.toplevel,
           analyze_options=' '.join(analyze_options),
           run_options=' '.join(run_options)))

        _vhdltypes = ("vhdlSource", "vhdlSource-87", "vhdlSource-93", "vhdlSource-2008")
        for f in src_files:
            if f.file_type in _vhdltypes:
                lib = ""
                if f.logical_name:
                    lib = ' --work='+f.logical_name
                makefile.write("\tghdl -a $(STD) $(ANALYZE_OPTIONS){lib} {file}\n".format(lib=lib, file=f.name))
            elif f.file_type in ["user"]:
                pass
            else:
                _s = "{} has unknown file type '{}'"
                logger.warning(_s.format(f.name, f.file_type))
        makefile.close()

    def run_main(self):
        cmd = 'make'
        args = ['run']

        if self.vlogparam:
            extra_options='EXTRA_OPTIONS='
            for k,v in self.vlogparam.items():
                extra_options += ' -g{}={}'.format(k,self._param_value_str(v,'"'))
            args.append(extra_options)
        self._run_tool(cmd, args)



from importlib import import_module
from os.path import dirname
from pkgutil import walk_packages

def get_edatool(name):
    return getattr(import_module('{}.{}'.format(__name__, name)),
                   name.capitalize())

def get_edatools():
    return [get_edatool(pkg) for _, pkg, _ in walk_packages([dirname(__file__)])]



import tempfile
import os.path

from fusesoc.config import Config

from test_common import cores_root, cache_root, library_root

build_root = 'test_build_root'

EXAMPLE_CONFIG = """
[main]
build_root = {build_root}
cache_root = {cache_root}
cores_root = {cores_root}
library_root = {library_root}

[library.test_lib]
location = {library_root}/test_lib
auto-sync = false
sync-uri = https://github.com/fusesoc/fusesoc-cores
"""

def test_config_file():
    tcf = tempfile.TemporaryFile(mode="w+")
    tcf.write(EXAMPLE_CONFIG.format(
            build_root = build_root,
            cache_root = cache_root,
            cores_root = cores_root,
            library_root = library_root
            )
        )
    tcf.seek(0)

    conf = Config(file=tcf)

    assert conf.build_root == build_root

def test_config_path():
    tcf = tempfile.NamedTemporaryFile(mode="w+")
    tcf.write(EXAMPLE_CONFIG.format(
            build_root = build_root,
            cache_root = cache_root,
            cores_root = cores_root,
            library_root = library_root
            )
        )
    tcf.seek(0)

    conf = Config(path=tcf.name)

    assert conf.library_root == library_root

def test_config_failure():
    tcf = tempfile.NamedTemporaryFile(mode="w+")
    tcf.write(EXAMPLE_CONFIG.format(
            build_root = build_root,
            cache_root = cache_root,
            cores_root = "INCORRECT",
            library_root = library_root
            )
        )
    tcf.seek(0)

    conf = Config(path=tcf.name)

    assert not conf.cores_root == cores_root

def test_config_libraries():
    tcf = tempfile.NamedTemporaryFile(mode="w+")
    tcf.write(EXAMPLE_CONFIG.format(
            build_root = build_root,
            cache_root = cache_root,
            cores_root = cores_root,
            library_root = library_root
            )
        )
    tcf.seek(0)

    conf = Config(path=tcf.name)

    assert conf.libraries['test_lib']['location'] == os.path.join(library_root, 'test_lib')
    assert conf.libraries['test_lib']['sync-uri'] == 'https://github.com/fusesoc/fusesoc-cores'
    assert not conf.libraries['test_lib']['auto-sync']




import pytest

def test_copyto():
    import os
    import tempfile

    from fusesoc.edalizer import Edalizer
    from fusesoc.core import Core

    core = Core(os.path.join(os.path.dirname(__file__), 'cores', 'misc', 'copytocore.core'))
    flags = {'tool' : 'icarus'}

    work_root = tempfile.mkdtemp(prefix='copyto_')

    eda_api = Edalizer(core.name, flags, [core], work_root, None).edalize

    assert eda_api['files'] == [{'file_type': 'user',
                                 'logical_name': '',
                                 'name': 'copied.file',
                                 'is_include_file': False},
                                {'file_type': 'tclSource',
                                 'logical_name': '',
                                 'name': 'subdir/another.file',
                                 'is_include_file': False}]
    assert os.path.exists(os.path.join(work_root, 'copied.file'))
    assert os.path.exists(os.path.join(work_root, 'subdir', 'another.file'))

def test_export():
    import os
    import tempfile

    from fusesoc.edalizer import Edalizer
    from fusesoc.core import Core

    core = Core(os.path.join(os.path.dirname(__file__), 'cores', 'wb_intercon', 'wb_intercon-1.0.core'))

    build_root = tempfile.mkdtemp(prefix='export_')
    export_root = os.path.join(build_root, 'exported_files')
    eda_api = Edalizer(core.name,
                       {'tool' : 'icarus'},
                       [core],
                       work_root=os.path.join(build_root, 'work'),
                       export_root=export_root).edalize

    for f in [
            'wb_intercon_1.0/dummy_icarus.v',
            'wb_intercon_1.0/bench/wb_mux_tb.v',
            'wb_intercon_1.0/bench/wb_upsizer_tb.v',
            'wb_intercon_1.0/bench/wb_intercon_tb.v',
            'wb_intercon_1.0/bench/wb_arbiter_tb.v',
            'wb_intercon_1.0/rtl/verilog/wb_data_resize.v',
            'wb_intercon_1.0/rtl/verilog/wb_mux.v',
            'wb_intercon_1.0/rtl/verilog/wb_arbiter.v',
            'wb_intercon_1.0/rtl/verilog/wb_upsizer.v',
    ]:
        assert os.path.isfile(os.path.join(export_root, f))



import pytest
import shutil

def test_empty_eda_api():
    import tempfile
    from edalize import get_edatool

    (h, eda_api_file) = tempfile.mkstemp()

    with pytest.raises(RuntimeError):
        backend = get_edatool('icarus')(eda_api_file=eda_api_file)

def test_incomplete_eda_api():
    import tempfile
    import yaml
    from edalize import get_edatool

    (h, eda_api_file) = tempfile.mkstemp()
    contents = []
    
    with open(eda_api_file,'w') as f:
        f.write(yaml.dump({'version' : '0.1.2'}))

    with pytest.raises(RuntimeError) as excinfo:
        backend = get_edatool('icarus')(eda_api_file=eda_api_file)
    assert "Missing required parameter 'name'" in str(excinfo.value)

    with open(eda_api_file,'w') as f:
        f.write(yaml.dump({'version' : '0.1.2',
                           'name' : 'corename'}))

    backend = get_edatool('icarus')(eda_api_file=eda_api_file)

def test_eda_api_files():
    import tempfile
    import yaml

    from edalize import get_edatool
    files = [{'name' : 'plain_file'},
             {'name' : 'subdir/plain_include_file',
              'is_include_file' : True},
             {'name' : 'file_with_args',
              'file_type' : 'verilogSource',
              'logical_name' : 'libx'},
             {'name' : 'include_file_with_args',
              'is_include_file' : True,
              'file_type' : 'verilogSource',
              'logical_name' : 'libx'}]
    (h, eda_api_file) = tempfile.mkstemp(prefix='eda_api_files_')
    with open(eda_api_file,'w') as f:
        f.write(yaml.dump({'files' : files,
                           'name' : 'test_eda_api_files'}))

    backend = get_edatool('icarus')(eda_api_file=eda_api_file)
    (parsed_files, incdirs) = backend._get_fileset_files()

    assert len(parsed_files) == 2
    assert parsed_files[0].name         == 'plain_file'
    assert parsed_files[0].file_type    == ''
    assert parsed_files[0].logical_name == ''
    assert parsed_files[1].name         == 'file_with_args'
    assert parsed_files[1].file_type    == 'verilogSource'
    assert parsed_files[1].logical_name == 'libx'

    assert incdirs == ['subdir', '.']

def test_eda_api_hooks():
    import os.path
    import tempfile
    import yaml
    from edalize import get_edatool

    tests_dir = os.path.dirname(__file__)
    ref_dir   = os.path.join(tests_dir, __name__)

    script = 'exit_1_script'
    hooks = {'pre_build' : [
        {'cmd' : ['sh', os.path.join(ref_dir, script)],
         'name' : script}]}

    (h, eda_api_file) = tempfile.mkstemp(prefix='eda_api_hooks_')
    with open(eda_api_file,'w') as f:
        f.write(yaml.dump({'hooks' : hooks,
                           'name' : script}))

    backend = get_edatool('icarus')(eda_api_file=eda_api_file)
    with pytest.raises(RuntimeError):
        backend.build()



import pytest

def test_quartus():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['vlogdefine', 'vlogparam']
    name         = 'test_quartus_0'
    tool         = 'quartus'
    tool_options = {
        'family'          : 'Cyclone V',
        'device'          : '5CSXFC6D6F31C8ES',
        'quartus_options' : ['some', 'quartus_options'],
    }

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options)
    backend.configure(args)

    compare_files(ref_dir, work_root, ['Makefile',
                                       'config.mk',
                                       name+'.tcl'])

    backend.build()
    compare_files(ref_dir, work_root, [
        'ip-generate.cmd',
        'quartus_asm.cmd',
        'quartus_fit.cmd',
        'quartus_sh.cmd',
        'quartus_sta.cmd',
    ])



from functools import total_ordering

@total_ordering
class Vlnv(object):
    def __init__(self, s, default_relation = ">="):
        def _is_rev(s):
            return s.startswith('r') and s[1:].isdigit()
        def _is_version(s):
            return s[0].isdigit()

        if s.startswith('!'):
            self.conflict = True
            _s = s[1:]
        else:
            self.conflict = False
            _s = s[:]
        if _s[0:2] in ['>=', '<=']:
            self.relation = _s[0:2]
            _s = _s[2:]
        elif s[0] in ['>', '<']:
            self.relation = s[0]
            _s = _s[1:]
        elif s[0] in ['=']:
            self.relation = "=="
            _s = _s[1:]
        else:
            self.relation = ""

        vlnv_parts = _s.split(':')

        self.revision = 0
        #legacy naming. Only name
        if len(vlnv_parts) == 1:
            self.vendor  = ""
            self.library = ""
            sl = vlnv_parts[0].rsplit('-')
            if len(sl) == 1:
                #Simplest case. No '-' => Only name
                self.name = s
                self.version = ""
            else:
                #If last part is the revision, save and pop from list
                if _is_rev(sl[-1]):
                    self.revision = int(sl.pop()[1:])

                #If last part is version, save and pop from list
                if len(sl) > 1 and _is_version(sl[-1]):
                    self.version = sl.pop()
                else:
                    self.version = ""

                self.name    = '-'.join(sl)

        #No version tag
        elif len(vlnv_parts) == 3:
            self.vendor  = vlnv_parts[0]
            self.library = vlnv_parts[1]
            self.name    = vlnv_parts[2]
            self.version = ""
        #Full vlnv
        elif len(vlnv_parts) == 4:
            self.vendor  = vlnv_parts[0]
            self.library = vlnv_parts[1]
            self.name    = vlnv_parts[2]
            sl = vlnv_parts[3].split('-')
            if len(sl) > 1 and _is_rev(sl[-1]):
                self.revision = int(sl.pop()[1:])
                self.version = '-'.join(sl)
            else:
                self.version = vlnv_parts[3]
        else:
            raise SyntaxError("Illegal core name '{}'".format(s))

        if self.version or (self.revision > 0):
            if not self.relation:
                # Version specified without relational operator
                # Assume user wants the exact version
                self.relation = "=="
            if not self.version:
                self.version = "0"
        else:
            if self.relation:
                _s = "{}: '{}' operator requires a version "
                raise SyntaxError(_s.format(s, self.relation))
            #No version specifier means any version i.e. >=0
            self.version = "0"
            self.relation = default_relation

        #Create sanitized name
        self.sanitized_name = str(self).lstrip(':').replace(":", "_")

    def __str__(self):
        if self.relation == '=':
            relation = ""
        else:
            relation = self.relation
        if self.revision > 0:
            revision = "-r"+str(self.revision)
        else:
            revision = ""
#        return "{}{}:{}:{}:{}{}".format(relation,
        return "{}{}:{}:{}:{}{}".format("",
                                        self.vendor,
                                        self.library,
                                        self.name,
                                        self.version,
                                        revision)
    def depstr(self):
        if self.relation == '==':
            relation = ""
        else:
            relation = self.relation
        return relation+str(self)

    def __eq__(self, other):
        return ((self.vendor , self.library , self.name , self.version) ==
                (other.vendor, other.library, other.name, other.version))

    def __lt__(self, other):
        return ((self.vendor , self.library , self.name , self.version) <
                (other.vendor, other.library, other.name, other.version))



import difflib
import os
import shutil
import pytest

from edalize_common import compare_files

tool      = 'verilator'
tests_dir = os.path.dirname(__file__)
ref_dir   = os.path.join(tests_dir, __name__)
core_name = 'mor1kx-generic_0'

cmdlineargs = ' --cmdlinearg_bool --cmdlinearg_int=42 --cmdlinearg_str=hello'.split()
vlogdefines = ' --vlogdefine_bool --vlogdefine_int=42 --vlogdefine_str=hello'.split()
vlogparams  = '--vlogparam_bool --vlogparam_int=42 --vlogparam_str=hello'.split()
params    = vlogparams + vlogdefines + cmdlineargs

def test_verilator_configure():
    import os.path
    import tempfile
    from edalize import get_edatool

    for mode in ['cc', 'sc', 'lint-only']:
        work_root    = tempfile.mkdtemp()
        eda_api_file = os.path.join(ref_dir, mode, core_name) + '.eda.yml'

        backend = get_edatool(tool)(eda_api_file=eda_api_file, work_root=work_root)

        if mode is 'cc':
            _params = params
        else:
            _params = []
        backend.configure(_params)

        compare_files(ref_dir, work_root, ['Makefile'])

        compare_files(os.path.join(ref_dir, mode),
                      work_root,
                      ['config.mk', core_name+'.vc'])

def test_verilator_run():
    import os.path
    import tempfile
    from edalize import get_edatool
    ref_dir_cc = os.path.join(ref_dir, 'cc')

    work_root    = tempfile.mkdtemp()
    eda_api_file = os.path.join(ref_dir_cc, core_name)+ '.eda.yml'
    backend = get_edatool(tool)(eda_api_file=eda_api_file, work_root=work_root)
    dummy_exe = 'V'+backend.tool_options['top_module']
    shutil.copy(os.path.join(ref_dir, dummy_exe),
                os.path.join(work_root, dummy_exe))

    backend.run(params)

    compare_files(ref_dir, work_root, ['run.cmd'])



import logging
import os.path
from fusesoc.provider.provider import Provider

logger = logging.getLogger(__name__)

class Local(Provider):
    @staticmethod
    def init_library(library):
        if not os.path.isdir(library['location']):
            logger.error("Local library at location '{}' not found.".format(library['location']))
            exit(1)

    def _checkout(self, local_dir):
        pass

    @staticmethod
    def update_library(library):
        pass



import os
import logging

from edalize.edatool import Edatool

logger = logging.getLogger(__name__)

class Xsim(Edatool):

    tool_options = {'lists' : {'xelab_options' : 'String',
                               'xsim_options'  : 'String'}}

    argtypes = ['plusarg', 'vlogdefine', 'vlogparam']

    MAKEFILE_TEMPLATE="""#Auto generated by Edalize
include config.mk

all: xsim.dir/$(TARGET)/xsimk

xsim.dir/$(TARGET)/xsimk:
	xelab $(TOPLEVEL) -prj $(TARGET).prj -snapshot $(TARGET) $(VLOG_DEFINES) $(VLOG_INCLUDES) $(VLOG_PARAMS) $(XELAB_OPTIONS)

run: xsim.dir/$(TARGET)/xsimk
	xsim --tclbatch run.tcl $(XSIM_OPTIONS) $(TARGET) $(EXTRA_OPTIONS)

run-gui: xsim.dir/$(TARGET)/xsimk
	xsim --gui --tclbatch run-gui.tcl $(XSIM_OPTIONS) $(TARGET) $(EXTRA_OPTIONS)
"""

    CONFIG_MK_TEMPLATE = """#Auto generated by Edalize
TARGET        = {target}
TOPLEVEL      = {toplevel}

VLOG_DEFINES  = {vlog_defines}
VLOG_INCLUDES = {vlog_includes}
VLOG_PARAMS   = {vlog_params}

XELAB_OPTIONS =	{xelab_options}
XSIM_OPTIONS  = {xsim_options}
"""

    RUN_TCL_TEMPLATE = """#Auto generated by Edalize
run all
quit
"""

    RUN_GUI_TCL_TEMPLATE = """#Auto generated by Edalize
add_wave -radix hex /
run all
"""

    def configure_main(self):
        self._write_config_files()

        #Check if any VPI modules are present and display warning
        if len(self.vpi_modules) > 0:
            modules = [m['name'] for m in self.vpi_modules]
            logger.error('VPI modules not supported by Xsim: %s' % ', '.join(modules))

    def _write_config_files(self):
        with open(os.path.join(self.work_root, self.name+'.prj'),'w') as f:
            (src_files, self.incdirs) = self._get_fileset_files()
            for src_file in src_files:
                cmd = ""
                if src_file.file_type.startswith("verilogSource"):
                    cmd = 'verilog'
                elif src_file.file_type == 'vhdlSource-2008':
                    cmd = 'vhdl2008'
                elif src_file.file_type.startswith("vhdlSource"):
                    cmd = 'vhdl'
                elif src_file.file_type.startswith("systemVerilogSource"):
                    cmd = 'sv'
                elif src_file.file_type in ["user"]:
                    pass
                else:
                    _s = "{} has unknown file type '{}'"
                    logger.warning(_s.format(src_file.name, src_file.file_type))
                if cmd:
                    if src_file.logical_name:
                        lib = src_file.logical_name
                    else:
                        lib = 'work'
                    f.write('{} {} {}\n'.format(cmd, lib, src_file.name))

        with open(os.path.join(self.work_root, 'config.mk'), 'w') as f:
            vlog_defines  = ' '.join(['--define {}={}'.format(k,v) for k,v, in self.vlogdefine.items()])
            vlog_includes = ' '.join(['-i '+k for k in self.incdirs])
            vlog_params   = ' '.join(['--generic_top {}={}'.format(k, self._param_value_str(v)) for k,v, in self.vlogparam.items()])
            xelab_options = ' '.join(self.tool_options.get('xelab_options', []))
            xsim_options  = ' '.join(self.tool_options.get('xsim_options' , []))

            f.write(self.CONFIG_MK_TEMPLATE.format(target=self.name,
                                                   toplevel=self.toplevel,
                                                   vlog_defines = vlog_defines,
                                                   vlog_includes = vlog_includes,
                                                   vlog_params   = vlog_params,
                                                   xelab_options = xelab_options,
                                                   xsim_options  = xsim_options))

        with open(os.path.join(self.work_root, 'run.tcl'), 'w') as f:
            f.write(self.RUN_TCL_TEMPLATE)
        with open(os.path.join(self.work_root, 'run-gui.tcl'), 'w') as f:
            f.write(self.RUN_GUI_TCL_TEMPLATE)
        with open(os.path.join(self.work_root, 'Makefile'), 'w') as f:
            f.write(self.MAKEFILE_TEMPLATE)

    def run_main(self):
        args = ['run']
        # Plusargs
        if self.plusarg:
            _s = '--testplusarg {}={}'
            args.append('EXTRA_OPTIONS='+' '.join([_s.format(k, v) for k,v in self.plusarg.items()]))

        self._run_tool('make', args)



from collections import OrderedDict
import importlib
import logging
import os
import shutil

from ipyxact.ipyxact import Component
from fusesoc import utils
from fusesoc.provider import get_provider
from fusesoc.vlnv import Vlnv
from fusesoc.capi1 import section
from fusesoc.capi1.fusesocconfigparser import FusesocConfigParser
from fusesoc.capi1.plusargs import Plusargs

logger = logging.getLogger(__name__)

class FileSet(object):
    def __init__(self, name="", file=[], usage = [], private=False):
        self.name    = name
        self.file    = file
        self.usage   = usage
        self.private = private

    def __str__(self):
        s = """Name  : {}
 Scope : {}
 Usage : {}
 Files :\n""".format(self.name,
                     "private" if self.private else "public",
                     '/'.join(self.usage))
        if not self.file:
            s += " <No files>\n"
        else:
            _longest_name = max([len(x.name) for x in self.file])
            _longest_type = max([len(x.file_type) for x in self.file])
            for f in self.file:
                _s = "  {} {} {}\n"
                s += _s.format(f.name.ljust(_longest_name),
                               f.file_type.ljust(_longest_type),
                               "(include file)" if f.is_include_file else "")
        return s

class Core:
    def __init__(self, core_file, cache_root='', build_root=''):
        basename = os.path.basename(core_file)
        self.depend = []
        self.simulators = []

        self.plusargs = None
        self.provider = None
        self.backend  = None

        for s in section.SECTION_MAP:
            assert(not hasattr(self, s))
            if(section.SECTION_MAP[s].named):
                setattr(self, s, OrderedDict())
            else:
                setattr(self, s, None)

        self.core_root = os.path.dirname(core_file)
        self.files_root = self.core_root
        self.build_root = build_root

        self.export_files = []

        config = FusesocConfigParser(core_file)

        #Add .system options to .core file
        system_file = os.path.join(self.core_root, basename.split('.core')[0]+'.system')
        if os.path.exists(system_file):
            self._merge_system_file(system_file, config)

        #FIXME : Make simulators part of the core object
        self.simulator        = config.get_section('simulator')
        if not 'toplevel' in self.simulator:
            self.simulator['toplevel'] = 'orpsoc_tb'

        for s in section.load_all(config, core_file):
            if type(s) == tuple:
                _l = getattr(self, s[0].TAG)
                _l[s[1]] = s[0]
                setattr(self, s[0].TAG, _l)
            else:
                setattr(self, s.TAG, s)

        if not self.main:
            self.main = section.MainSection()
        if self.main.name:
            self.name = Vlnv(self.main.name)
        else:
            self.name = Vlnv(basename.split('.core')[0])

        self.sanitized_name = self.name.sanitized_name

        self.depend     = self.main.depend[:]

        self.simulators = self.main.simulators

        if self.main.backend:
            try:
                self.backend = getattr(self, self.main.backend)
            except AttributeError:
                raise SyntaxError('Invalid backend "{}"'.format(self.main.backend))

        self._collect_filesets()

        cache_root = os.path.join(cache_root, self.sanitized_name)
        if config.has_section('plusargs'):
            self._warning("plusargs section is deprecated and will not be parsed by FuseSoC. Please migrate to parameters")
            self.plusargs = Plusargs(dict(config.items('plusargs')))
        if config.has_section('verilator') and config.has_option('verilator', 'define_files'):
            self._warning("verilator define_files are deprecated")
        if config.has_section('provider'):
            items    = dict(config.items('provider'))
            patch_root = os.path.join(self.core_root, 'patches')
            patches = self.main.patches
            if os.path.exists(patch_root):
                for p in sorted(os.listdir(patch_root)):
                    patches.append(os.path.join('patches', p))
            items['patches'] = patches
            provider_name = items.get('name')
            if provider_name is None:
                raise RuntimeError('Missing "name" in section [provider]')
            self.provider = get_provider(provider_name)(
                items, self.core_root, cache_root)
        if self.provider:
            self.files_root = self.provider.files_root

        # We need the component file here, but it might not be
        # available until the core is fetched. Try to fetch first if any
        # of the component files are missing
        if False in [os.path.exists(f) for f in self.main.component]:
            self.setup()

        for f in self.main.component:
            self._parse_component(f)

    def cache_status(self):
        if self.provider:
            return self.provider.status()
        else:
            return 'local'

    def get_depends(self, flags={}):
        self._debug("Getting dependencies for flags {}".format(str(flags)))
        _depends = self.depend
        try:
            _depends += getattr(self, flags['tool']).depend
        except (AttributeError, KeyError):
            pass
        return _depends

    def get_files(self, flags={}):
        files = []
        flow = self._get_flow(flags)
        usage = set([flow, flags['tool']])

        for fs in self.file_sets:
            if (not fs.private or flags['is_toplevel']) and (usage & set(fs.usage)):
                files += fs.file
        return files

    def get_parameters(self, flags={}):
        self._debug("Getting parameters for flags '{}'".format(str(flags)))
        parameters = {}
        for k, v in self.parameter.items():
            if (v.scope == 'public') or flags['is_toplevel']:
                parameters[k] = {'datatype'  : v.datatype,
                                 'paramtype' : v.paramtype}
                for field in ['default','description']:
                    if getattr(v, field):
                        parameters[k][field] = getattr(v, field)
        self._debug("Found parameters {}".format(parameters))
        return parameters

    def get_scripts(self, files_root, flags):
        def _build_dict(v, env):
            return [{'name' : x, 'cmd' : ['sh', os.path.join(files_root, x)], 'env' : env} for x in v]
        scripts = {}
        if self.scripts:
            env = {'BUILD_ROOT' : self.build_root,
                   'FILES_ROOT' : files_root,
            }
            flow = self._get_flow(flags)
            if flow is 'sim':
                for s in ['pre_build_scripts', 'pre_run_scripts', 'post_run_scripts']:
                    v = getattr(self.scripts, s)
                    if v:
                        scripts[s[0:-8]] = _build_dict(v, env)
            #For backwards compatibility we only use the script from the
            #top-level core in synth flows. We also rename them here to match
            #the backend stages and set the SYSTEM_ROOT env var
            elif flow is 'synth' and flags['is_toplevel']:
                env['SYSTEM_ROOT'] = self.files_root
                v = self.scripts.pre_synth_scripts
                if v:
                    scripts['pre_build'] = _build_dict(v, env)
                v = self.scripts.post_impl_scripts
                if v:
                    scripts['post_build'] = _build_dict(v, env)
        return scripts

    def get_toplevel(self, flags={}):
        self._debug("Getting toplevel for flags {}".format(str(flags)))
        if flags['tool'] == 'verilator':
            toplevel = self.verilator.top_module
        elif self.backend and self._get_flow(flags) == 'synth':
            toplevel = self.backend.top_module
        elif 'testbench' in flags and flags['testbench']:
            toplevel = flags['testbench']
        else:
            toplevel = self.simulator['toplevel']
        self._debug("Matched toplevel {}".format(toplevel))
        return toplevel

    def get_tool(self, flags):
        self._debug("Getting tool for flags {}".format(str(flags)))
        tool = None
        flow = self._get_flow(flags)
        if flags['tool']:
            tool =  flags['tool']
        elif flags['target'] == 'synth':
            if hasattr(self.main, 'backend'):
                tool = self.main.backend
        else:
            if len(self.simulators) > 0:
                tool = self.simulators[0]
        self._debug(" Matched tool {}".format(tool))
        return tool

    def get_tool_options(self, flags):
        self._debug("Getting tool options for flags {}".format(str(flags)))
        options = {}
        if hasattr(self, flags['tool']):
            section = getattr(self, flags['tool'])
        else:
            self._debug("CAPI1 does not support tool '{}'".format(flags['tool']))
            section = None

        if section:

            #Special cases for verilator
            if flags['tool'] == 'verilator':
                #Pick up verilator libs from all dependencies
                options['libs'] = section.libs

                #Set the mode option
                if flags['is_toplevel']:
                    if self.verilator.source_type == 'systemC':
                        options['mode'] = 'sc'
                    else:
                        options['mode'] = 'cc'
                    del(self.verilator.source_type)

                if self.verilator.cli_parser == 'fusesoc':
                    self.verilator.cli_parser = 'managed'
                elif self.verilator.cli_parser == '':
                    self.verilator.cli_parser = 'passthrough'

            #Otherwise, only care about options from toplevel core
            if flags['is_toplevel']:

                #Special case for isim. isim_options are really fuse_options
                if flags['tool'] == 'isim':
                    section._members['fuse_options'] = section._members.pop('isim_options')
                    section.fuse_options = section.isim_options

                #Special case for xsim. xsim_options are really xelab_options
                #Also add flags that were previously hardcoded in the backend
                if flags['tool'] == 'xsim':
                    if 'xsim_options' in section._members:
                        section._members['xelab_options'] = section._members.pop('xsim_options')
                        section.xelab_options = ['--timescale 1ps/1ps',
                                                 '--debug typical']
                        section.xelab_options += section.xsim_options
                    
                for member in section._members:
                    if hasattr(section, member) and getattr(section, member) and not member == 'depend':
                        #Strip quoted strings
                        _member = getattr(section, member)
                        if (type(_member) == str) and _member.startswith('"') and _member.endswith('"'):
                            _member = _member[1:-1]
                        options[member] = _member
        self._debug("Found tool options {}".format(str(options)))
        return options

    def get_vpi(self, flags):
        self._debug("Getting VPI libraries for flags {}".format(flags))
        vpi = []
        if self.vpi:
            vpi.append({'name'         : self.sanitized_name,
                        'src_files'    : [f.name for f in self.vpi.src_files],
                        'include_dirs' : self.vpi.include_dirs,
                        'libs'         : [l[2:] for l in self.vpi.libs],
            })
        self._debug(" Matched VPI libraries {}".format([v['name'] for v in vpi]))
        return vpi

    def get_work_root(self, flags):
        if self._get_flow(flags) is 'synth':
            s = 'bld-'
        else:
            s = 'sim-'
        return s+flags['tool']

    def setup(self):
        if self.provider:
            self.provider.fetch()

    def export(self, dst_dir, flags={}):
        if os.path.exists(dst_dir):
            shutil.rmtree(dst_dir)


        src_files = [f.name for f in self.get_files(flags)]
        if self.vpi and flags['tool'] in ['icarus', 'modelsim', 'rivierapro']:
            src_files += [f.name for f in self.vpi.src_files + self.vpi.include_files]
        for section in self.get_scripts(dst_dir, flags).values():
            for script in section:
                src_files.append(script['name'])

        self._debug("Exporting {}".format(str(src_files)))

        dirs = list(set(map(os.path.dirname,src_files)))
        for d in dirs:
            if not os.path.exists(os.path.join(dst_dir, d)):
                os.makedirs(os.path.join(dst_dir, d))

        for f in src_files:
            if not os.path.isabs(f):
                if(os.path.exists(os.path.join(self.core_root, f))):
                    shutil.copy2(os.path.join(self.core_root, f),
                                    os.path.join(dst_dir, f))
                elif (os.path.exists(os.path.join(self.files_root, f))):
                    shutil.copy2(os.path.join(self.files_root, f),
                                    os.path.join(dst_dir, f))
                else:
                    raise RuntimeError('Cannot find %s in :\n\t%s\n\t%s'
                                  % (f, self.files_root, self.core_root))

    def _get_flow(self, flags):
        flow = None
        if 'tool' in flags:
            if flags['tool'] in ['ghdl', 'icarus', 'isim', 'modelsim', 'rivierapro', 'xsim']:
                flow = 'sim'
            elif flags['tool'] in ['icestorm', 'ise', 'quartus', 'verilator', 'vivado', 'spyglass']:
                flow = 'synth'
        elif 'target' in flags:
            if flags['target'] is 'synth':
                flow = 'synth'
            elif flags['target'] is 'sim':
                flow = 'sim'
        return flow

    def _merge_system_file(self, system_file, config):
        def _replace(sec, src=None, dst=None):
            if not system.has_section(sec):
                return

            if not config.has_section(sec):
                config.add_section(sec)

            if src:
                if system.has_option(sec, src):
                    items = [src]
                else:
                    items = []
            else:
                items = system.options(sec)

            for item in items:
                if dst:
                    _dst = dst
                else:
                    _dst = item
                if not config.has_option(sec, _dst):
                    config.set(sec, _dst, system.get(sec, item))

        system = FusesocConfigParser(system_file)
        for section in ['icestorm', 'ise', 'quartus', 'vivado']:
            _replace(section)

        _replace('main', 'backend')
        _replace('scripts', 'pre_build_scripts' , 'pre_synth_scripts')
        _replace('scripts', 'post_build_scripts', 'post_impl_scripts')

    def _collect_filesets(self):
        def _append_files(section, file_type, is_include_file=False):
            _files = []
            for f in section:
                if not f.file_type:
                    f.file_type = file_type
                f.is_include_file = is_include_file
                _files.append(f)
            return _files

        self.file_sets = []

        _v = self.verilog
        if _v:
            _files  = _append_files(_v.include_files, _v.file_type, True)
            _files += _append_files(_v.src_files, _v.file_type)
            self.file_sets.append(FileSet(name  = "verilog_src_files",
                                          file  = _files,
                                          usage = ['sim', 'synth']))

            _files  = _append_files(_v.tb_include_files, _v.file_type, True)
            _files += _append_files(_v.tb_src_files, _v.file_type)
            self.file_sets.append(FileSet(name  = "verilog_tb_src_files",
                                          file  = _files,
                                          usage = ['sim']))
            _files  = _append_files(_v.tb_private_src_files, _v.file_type)
            self.file_sets.append(FileSet(name    = "verilog_tb_private_src_files",
                                          file    = _files,
                                          usage   = ['sim'],
                                          private = True))


        for k, v in self.fileset.items():
            self.file_sets.append(FileSet(name = k,
                                          file = v.files,
                                          usage = v.usage,
                                          private = (v.scope == 'private')))

        _bname = self.main.backend
        _b = self.backend
        if _b and _bname in ['icestorm', 'ise', 'quartus']:
            _files = []
            if _bname == 'icestorm':
                _files += _append_files(_b.pcf_file, 'PCF')
                del(_b.pcf_file)
            elif _bname == 'ise':
                _files += _append_files(_b.tcl_files, 'tclSource')
                _files += _append_files(_b.ucf_files, 'UCF')
                del(_b.ucf_files)
            elif _bname == 'quartus':
                _files += _append_files(_b.qsys_files, 'QSYS')
                _files += _append_files(_b.sdc_files, 'SDC')
                _files += _append_files(_b.tcl_files, 'tclSource')
                del(_b.qsys_files)
                del(_b.sdc_files)
                del(_b.tcl_files)
            if _files:
                self.file_sets.append(FileSet(name = "backend_files",
                                              file = _files,
                                              usage = [_bname],
                                              private = True))
                self.export_files += [f.name for f in _files]
        if self.verilator:
            if self.verilator.source_type == 'CPP':
                _file_type = 'cppSource'
            elif self.verilator.source_type == 'systemC':
                _file_type = 'systemCSource'
            elif self.verilator.source_type == 'C':
                _file_type = 'cSource'
            else:
                raise RuntimeError("Invalid verilator file type '{}'".format(self.verilator.source_type))
            _files  = _append_files(self.verilator.src_files, _file_type)
            _files += _append_files(self.verilator.include_files, _file_type, True)
            del(self.verilator.src_files)
            self.file_sets.append(FileSet(name = "verilator_src_files",
                                          file = _files,
                                          usage = ['verilator']))
            self.export_files += [f.name for f in _files]

            _files  = _append_files(self.verilator.tb_toplevel, _file_type)
            self.file_sets.append(FileSet(name = "verilator_tb_toplevel",
                                          file = _files,
                                          usage = ['verilator'],
                                          private = True))
            del(self.verilator.tb_toplevel)
            self.export_files += [f.name for f in _files]

    def _debug(self, msg):
        logger.debug("{} : {}".format(str(self.name), msg))

    def _warning(self, msg):
        logger.warning("{} : {}".format(str(self.name), msg))

    def _parse_component(self, component_file):
        component_dir = os.path.dirname(component_file)
        component = Component()
        component.load(os.path.join(self.files_root, component_file))

        if not self.main.description:
            self.main.description = component.description

        _file_sets = []
        for file_set in component.fileSets.fileSet:
            _name = file_set.name
            for f in file_set.file:
                f.name = os.path.normpath(os.path.join(component_dir, f.name))
                self.export_files.append(f.name)
                #FIXME: Harmonize underscore vs camelcase
                f.file_type = f.fileType
                if f.isIncludeFile == 'true':
                    f.is_include_file = True
                else:
                    f.is_include_file = False
                f.logical_name = f.logicalName
                f.copyto = ""
            #FIXME: Handle duplicates. Resolution function? (merge/replace, prio ipxact/core)
            _taken = False
            for fs in self.file_sets:
                if fs.name == file_set.name:
                    _taken = True
            if not _taken:
                _file_sets.append(FileSet(name = file_set.name,
                                          file = file_set.file[:],
                                          usage = ['sim', 'synth']))
        self.file_sets += _file_sets
    def info(self):
        HEADER = """CORE INFO
Name                 : {}
Core root            : {}
Supported simulators : {}
Common dependencies  : {}\n\n"""

        s = HEADER.format(str(self.name),
                          self.core_root,
                          ' '.join(self.simulators),
                          ' '.join([x.depstr() for x in self.main.depend]))
        for sec in sorted(section.SECTION_MAP):
            if sec in ['main', 'verilog', 'fileset']:
                continue
            obj = getattr(self, sec)
            if obj:
                if(type(obj) == OrderedDict):
                    for k, v in obj.items():
                        s += "== {} {} ==\n{}\n".format(sec,k, v)
                else:
                    s += "== {} ==\n{}\n".format(sec, obj)
        s += "File sets:\n"
        for fs in self.file_sets:
            s += str(fs) + '\n'

        if self.main.backend:
            s += "\n== Backend {} ==\n{}\n".format(self.main.backend,
                                                   self.backend)
        return s



import pytest

def test_icestorm():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['vlogdefine', 'vlogparam']
    name         = 'test_icestorm_0'
    tool         = 'icestorm'
    tool_options = {
        'yosys_synth_options' : ['some', 'yosys_synth_options'],
        'arachne_pnr_options' : ['a', 'few', 'arachne_pnr_options'],
    }

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options)
    backend.configure(args)

    compare_files(ref_dir, work_root, ['Makefile',
                                       name+'.ys'])

    f = os.path.join(work_root, 'pcf_file.pcf')
    with open(f, 'a'):
        os.utime(f, None)

    backend.build()
    compare_files(ref_dir, work_root, ['yosys.cmd'])
    compare_files(ref_dir, work_root, ['arachne-pnr.cmd'])
    compare_files(ref_dir, work_root, ['icepack.cmd'])

def test_icestorm_minimal():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend_minimal, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__, 'minimal')
    name         = 'test_icestorm_0'
    tool         = 'icestorm'

    (backend, work_root) = setup_backend_minimal(name, tool, [{'name' : 'pcf_file.pcf', 'file_type' : 'PCF'}])
    backend.configure('')

    compare_files(ref_dir, work_root, ['Makefile',
                                       name+'.ys'])

    f = os.path.join(work_root, 'pcf_file.pcf')
    with open(f, 'a'):
        os.utime(f, None)

    backend.build()
    compare_files(ref_dir, work_root, ['yosys.cmd'])
    compare_files(ref_dir, work_root, ['arachne-pnr.cmd'])
    compare_files(ref_dir, work_root, ['icepack.cmd'])

def test_icestorm_no_pcf():
    import os

    from edalize_common import compare_files, setup_backend_minimal, tests_dir

    name         = 'test_icestorm_0'
    tool         = 'icestorm'

    (backend, work_root) = setup_backend_minimal(name, tool, [])
    with pytest.raises(RuntimeError) as e:
        backend.configure('')
    assert "Icestorm backend requires a PCF file" in str(e.value)

def test_icestorm_multiple_pcf():
    import os

    from edalize_common import compare_files, setup_backend_minimal, tests_dir

    name         = 'test_icestorm_0'
    tool         = 'icestorm'

    (backend, work_root) = setup_backend_minimal(name, tool, [{'name' : 'pcf_file.pcf', 'file_type' : 'PCF'},
                                                              {'name' : 'pcf_file2.pcf', 'file_type' : 'PCF'}])
    with pytest.raises(RuntimeError) as e:
        backend.configure('')
    assert "Icestorm backend supports only one PCF file. Found pcf_file.pcf, pcf_file2.pcf" in str(e.value)

def test_icestorm_nextpnr():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__, 'nextpnr')
    paramtypes   = ['vlogdefine', 'vlogparam']
    name         = 'test_icestorm_0'
    tool         = 'icestorm'
    tool_options = {
        'yosys_synth_options' : ['some', 'yosys_synth_options'],
        'arachne_pnr_options' : ['a', 'few', 'arachne_pnr_options'],
        'nextpnr_options'     : ['multiple', 'nextpnr_options'],
        'pnr'                 : 'next',
    }

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options)
    backend.configure(args)

    compare_files(ref_dir, work_root, ['Makefile',
                                       name+'.ys'])

    f = os.path.join(work_root, 'pcf_file.pcf')
    with open(f, 'a'):
        os.utime(f, None)

    backend.build()
    compare_files(ref_dir, work_root, ['yosys.cmd'])
    compare_files(ref_dir, work_root, ['nextpnr-ice40.cmd'])
    compare_files(ref_dir, work_root, ['icepack.cmd'])

def test_icestorm_invalid_pnr():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['vlogdefine', 'vlogparam']
    name         = 'test_icestorm_0'
    tool         = 'icestorm'
    tool_options = {
        'pnr'                 : 'invalid',
    }

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options)
    with pytest.raises(RuntimeError) as e:
        backend.configure(args)
    assert "nvalid pnr option 'invalid'. Valid values are 'arachne' for Arachne-pnr or 'next' for nextpnr" in str(e.value)



import logging
import os
from fusesoc.utils import Launcher
logger = logging.getLogger(__name__)

class Provider(object):
    def __init__(self, config, core_root, files_root):
        self.config = config
        self.core_root = core_root
        self.files_root = files_root

        self.cachable = not (config.get('cachable', '') == 'false')
        self.patches = config.get('patches', [])

    def clean_cache(self):
        if os.path.exists(self.files_root):
            shutil.rmtree(self.files_root)

    def fetch(self):
        status = self.status()
        if status == 'empty':
            self._checkout(self.files_root)
            _fetched = True
        elif status == 'outofdate':
            self.clean_cache()
            self._checkout(self.files_root)
            _fetched = True
        elif status == 'downloaded':
            _fetched = False
        else:
            raise RuntimeError("Provider status is: '" + status + "'. This shouldn't happen")
        if _fetched:
            self._patch()

    def _patch(self):
        for f in self.patches:
            patch_file = os.path.abspath(os.path.join(self.core_root, f))
            if os.path.isfile(patch_file):
                logger.debug("  applying patch file: " + patch_file + "\n" +
                             "                   to: " + os.path.join(self.files_root))
                try:
                    Launcher('git', ['apply', '--unsafe-paths',
                                     '--directory', self.files_root,
                                     patch_file]).run()
                except OSError:
                    raise RuntimeError("Failed to call 'git' for patching core")

    def status(self):
        if not self.cachable:
            return 'outofdate'
        if not os.path.isdir(self.files_root):
            return 'empty'
        else:
            return 'downloaded'



import pytest

def test_icarus():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['plusarg', 'vlogdefine', 'vlogparam']
    name         = 'test_icarus_0'
    tool         = 'icarus'
    tool_options = {
        'iverilog_options' : ['some', 'iverilog_options'],
        'timescale'        : '1ns/1ns',
    }

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options, use_vpi=True)
    backend.configure(args)

    compare_files(ref_dir, work_root, ['Makefile',
                                       name+'.scr',
                                       'timescale.v',
    ])

    backend.build()
    compare_files(ref_dir, work_root, ['iverilog.cmd'])
    compare_files(ref_dir, work_root, ['iverilog-vpi.cmd'])

    backend.run(args)

    compare_files(ref_dir, work_root, ['vvp.cmd'])

def test_icarus_minimal():
    import os
    import shutil
    import tempfile
    import yaml

    from edalize import get_edatool

    from edalize_common import compare_files, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__, 'minimal')
    os.environ['PATH'] = os.path.join(tests_dir, 'mock_commands')+':'+os.environ['PATH']
    tool = 'icarus'
    name = 'test_'+tool+'_minimal_0'
    work_root = tempfile.mkdtemp(prefix=tool+'_')

    eda_api_file = os.path.join(work_root, name+'.eda.yml')
    with open(eda_api_file,'w') as f:
        f.write(yaml.dump({'name'         : name,
                           'toplevel' : 'top'}))

    backend = get_edatool(tool)(eda_api_file=eda_api_file)
    backend.configure([])

    compare_files(ref_dir, work_root, ['Makefile',
                                       name+'.scr',
    ])

    backend.build()
    compare_files(ref_dir, work_root, ['iverilog.cmd'])

    backend.run([])

    compare_files(ref_dir, work_root, ['vvp.cmd'])



import os
import logging

from edalize.edatool import Edatool

logger = logging.getLogger(__name__)

RUN_DO = """#Generated by Edalize
run -all
exit
"""
class Rivierapro(Edatool):

    tool_options = {'lists' : {'vlog_options' : 'String',
                               'vsim_options' : 'String'}}

    argtypes = ['plusarg', 'vlogdefine', 'vlogparam']

    def _write_build_rtl_tcl_file(self, tcl_main):
        tcl_build_rtl  = open(os.path.join(self.work_root, "edalize_build_rtl.tcl"), 'w')

        (src_files, incdirs) = self._get_fileset_files()
        vlog_include_dirs = ['+incdir+'+d.replace('\\','/') for d in incdirs]

        libs = []
        for f in src_files:
            if not f.logical_name:
                f.logical_name = 'work'
            if not f.logical_name in libs:
                tcl_build_rtl.write("vlib {}\n".format(f.logical_name))
                libs.append(f.logical_name)
            if f.file_type.startswith("verilogSource") or \
               f.file_type.startswith("systemVerilogSource"):

                cmd = 'vlog'
                args = []

                args += self.tool_options.get('vlog_options', [])

                if f.file_type.startswith("verilogSource"):
                    if f.file_type.endswith("95"):
                        args.append('-v95')
                    elif f.file_type.endswith("2001"):
                        args.append('-v2k')
                    elif f.file_type.endswith("2005"):
                        args.append('-v2k5')
                else:
                    args += ['-sv']

                for k, v in self.vlogdefine.items():
                    args += ['+define+{}={}'.format(k,self._param_value_str(v))]

                args += vlog_include_dirs
            elif f.file_type.startswith("vhdlSource"):
                cmd = 'vcom'
                if f.file_type.endswith("-87"):
                    args = ['-87']
                if f.file_type.endswith("-93"):
                    args = ['-93']
                if f.file_type.endswith("-2008"):
                    args = ['-2008']
                else:
                    args = []
            elif f.file_type == 'tclSource':
                cmd = None
                tcl_main.write("do {}\n".format(f.name))
            elif f.file_type == 'user':
                cmd = None
            else:
                _s = "{} has unknown file type '{}'"
                logger.warning(_s.format(f.name, f.file_type))
                cmd = None
            if cmd:
                args += ['-quiet']
                args += ['-work', f.logical_name]
                args += [f.name.replace('\\','/')]
                tcl_build_rtl.write("{} {}\n".format(cmd, ' '.join(args)))

    def _write_run_tcl_file(self):
        tcl_launch = open(os.path.join(self.work_root, "edalize_launch.tcl"), 'w')

        #FIXME: Handle failures. Save stdout/stderr
        vpi_options = []
        for vpi_module in self.vpi_modules:
            vpi_options += ['-pli', vpi_module['name']]

        args = ['vsim']
        args += self.tool_options.get('vsim_options', [])
        args += vpi_options
        args += self.toplevel.split()

        # Plusargs
        for key, value in self.plusarg.items():
            args += ['+{}={}'.format(key, self._param_value_str(value))]
        #Top-level parameters
        for key, value in self.vlogparam.items():
            args += ['-g{}={}'.format(key, self._param_value_str(value))]
        tcl_launch.write(' '.join(args)+'\n')
        tcl_launch.close()

        tcl_run = open(os.path.join(self.work_root, "edalize_run.tcl"), 'w')
        tcl_run.write("do edalize_launch.tcl\n")
        tcl_run.write("run -all\n")
        tcl_run.write("exit\n")
        tcl_run.close()

    def _write_build_vpi_tcl_file(self):
        tcl_build_vpi = open(os.path.join(self.work_root, "edalize_build_vpi.tcl"), 'w')
        for vpi_module in self.vpi_modules:
            _name = vpi_module['name']
            _incs = ' '.join(['-I'+d for d in vpi_module['include_dirs']])
            _libs = ' '.join(['-l'+l for l in vpi_module['libs']])
            _options = "-std=c99"
            _srcs = ' '.join(vpi_module['src_files'])
            _s = 'ccomp -pli -o {}.so {} {} {} {}\n'.format(vpi_module['name'],
                                                      _incs,
                                                      _libs,
                                                      _options,
                                                      _srcs)

            tcl_build_vpi.write(_s)
        tcl_build_vpi.close()

    def configure_main(self):
        tcl_main = open(os.path.join(self.work_root, "edalize_main.tcl"), 'w')
        tcl_main.write("do edalize_build_rtl.tcl\n")

        self._write_build_rtl_tcl_file(tcl_main)
        if self.vpi_modules:
            self._write_build_vpi_tcl_file()
            tcl_main.write("do edalize_build_vpi.tcl\n")
        tcl_main.close()
        self._write_run_tcl_file()

    def build_pre(self):
        if not os.getenv('ALDEC_PATH'):
            raise RuntimeError("Environment variable ALDEC_PATH was not found. It should be set to Riviera Pro install path. Please source <Riviera Pro install path>/etc/setenv to set it")
        super(Rivierapro, self).build_pre()

    def build_main(self):
        args = ['-c', '-do', 'do edalize_main.tcl; exit']
        self._run_tool('vsim', args)

    def run_main(self):
        if not os.getenv('ALDEC_PATH'):
            raise RuntimeError("Environment variable ALDEC_PATH was not found. It should be set to Riviera Pro install path. Please source <Riviera Pro install path>/etc/setenv to set it")

        args = ['-c', '-quiet', '-do', 'edalize_run.tcl']
        self._run_tool('vsim', args)



import os.path

from edalize.edatool import Edatool

class Icestorm(Edatool):

    tool_options = {
        'lists' : {
            'arachne_pnr_options' : 'String',
            'nextpnr_options'     : 'String',
            'yosys_synth_options' : 'String',
        },
        'members': {
            'pnr' : 'String',
        }
    }

    argtypes = ['vlogdefine', 'vlogparam']

    def configure_main(self):
        # Write yosys script file
        (src_files, incdirs) = self._get_fileset_files()
        with open(os.path.join(self.work_root, self.name+'.ys'), 'w') as yosys_file:
            for key, value in self.vlogdefine.items():
                yosys_file.write("verilog_defines -D{}={}\n".format(key, self._param_value_str(value)))

            yosys_file.write("verilog_defaults -push\n")
            yosys_file.write("verilog_defaults -add -defer\n")
            if incdirs:
                yosys_file.write("verilog_defaults -add {}\n".format(' '.join(['-I'+d for d in incdirs])))

            pcf_files = []
            for f in src_files:
                if f.file_type in ['verilogSource']:
                    yosys_file.write("read_verilog {}\n".format(f.name))
                elif f.file_type == 'PCF':
                    pcf_files.append(f.name)
                elif f.file_type == 'user':
                    pass
            for key, value in self.vlogparam.items():
                _s = "chparam -set {} {} $abstract\{}\n"
                yosys_file.write(_s.format(key,
                                           self._param_value_str(value, '"'),
                                           self.toplevel))

            yosys_file.write("verilog_defaults -pop\n")
            yosys_file.write("synth_ice40")
            yosys_synth_options = self.tool_options.get('yosys_synth_options', [])
            for option in yosys_synth_options:
                yosys_file.write(' ' + option)
            yosys_file.write(" -blif {}.blif".format(self.name))
            if self.toplevel:
                yosys_file.write(" -top " + self.toplevel)
            yosys_file.write("\n")
            yosys_file.write("write_json {}.json\n".format(self.name))

        if not pcf_files:
            raise RuntimeError("Icestorm backend requires a PCF file")
        elif len(pcf_files) > 1:
            raise RuntimeError("Icestorm backend supports only one PCF file. Found {}".format(', '.join(pcf_files)))

        pnr = self.tool_options.get('pnr', 'arachne')
        if not pnr in ['arachne', 'next']:
            raise RuntimeError("Invalid pnr option '{}'. Valid values are 'arachne' for Arachne-pnr or 'next' for nextpnr".format(pnr))
        # Write Makefile
        arachne_pnr_options = self.tool_options.get('arachne_pnr_options', [])
        nextpnr_options     = self.tool_options.get('nextpnr_options', [])
        template_vars = {
            'name'                : self.name,
            'pcf_file'            : pcf_files[0],
            'pnr'                 : pnr,
            'arachne_pnr_options' : arachne_pnr_options,
            'nextpnr_options'     : nextpnr_options,
        }
        self.render_template('icestorm-makefile.j2',
                             'Makefile',
                             template_vars)



import logging
import sys

from fusesoc.provider.provider import Provider
from fusesoc.utils import Launcher, cygpath, is_mingw

logger = logging.getLogger(__name__)

REPO_PATH = 'http://opencores.org/ocsvn/{}/{}/{}'

class Opencores(Provider):

    def _checkout(self, local_dir):
        repo_name = self.config.get('repo_name')
        repo_path = REPO_PATH.format(repo_name,
                                     repo_name,
                                     self.config.get('repo_root'))
        revision_number  = self.config.get('revision')
        logger.info("Downloading " + repo_name + " from OpenCores")

        if is_mingw():
            logger.debug("Using cygpath translation")
            local_dir = cygpath(local_dir)

        Launcher('svn', ['co', '-q', '--no-auth-cache',
                         '-r', revision_number,
                         '--username', 'orpsoc',
                         '--password', 'orpsoc',
                         repo_path,
                         local_dir]).run()



import pytest

def test_xsim():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['plusarg', 'vlogdefine', 'vlogparam']
    name         = 'test_xsim_0'
    tool         = 'xsim'
    tool_options = {'xelab_options' : ['some', 'xelab_options'],
                    'xsim_options'  : ['a', 'few', 'xsim_options']}

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options)
    backend.configure(args)

    compare_files(ref_dir, work_root, ['config.mk',
                                       'Makefile',
                                       name+'.prj',
                                       'run-gui.tcl',
                                       'run.tcl'])

    backend.build()
    compare_files(ref_dir, work_root, ['xelab.cmd'])

    xsimkdir = os.path.join(work_root, 'xsim.dir', name)
    os.makedirs(xsimkdir)
    with open(os.path.join(xsimkdir, 'xsimk'), 'w') as f:
        f.write("I am a compiled simulation kernel\n")
    backend.run(args)

    compare_files(ref_dir, work_root, ['xsim.cmd'])



import pytest

def test_isim():
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir      = os.path.join(tests_dir, __name__)
    paramtypes   = ['plusarg', 'vlogdefine', 'vlogparam']
    name         = 'test_isim_0'
    tool         = 'isim'
    tool_options = {
        'fuse_options' : ['some', 'fuse_options'],
        'isim_options' : ['a', 'few', 'isim_options'],
    }

    (backend, args, work_root) = setup_backend(paramtypes, name, tool, tool_options)
    backend.configure(args)

    compare_files(ref_dir, work_root,
                  ['config.mk',
                   'Makefile',
                   'run_test_isim_0.tcl',
                   'test_isim_0.prj'])

    dummy_exe = 'test_isim_0'
    shutil.copy(os.path.join(ref_dir, dummy_exe),
                os.path.join(work_root, dummy_exe))

    backend.run([])

    compare_files(ref_dir, work_root, ['run.cmd'])



import os.path
import logging

from edalize.edatool import Edatool

logger = logging.getLogger(__name__)

class Quartus(Edatool):

    tool_options = {'members' : {'family' : 'String',
                                 'device' : 'String'},
                    'lists'   : {'quartus_options' : 'String'}}

    argtypes = ['vlogdefine', 'vlogparam']

    MAKEFILE_TEMPLATE = """#Auto generated by Edalize
all: sta

include config.mk

project: $(DESIGN_NAME).tcl qsys
	quartus_sh $(QUARTUS_OPTIONS) -t $(DESIGN_NAME).tcl

map: project
	quartus_map $(QUARTUS_OPTIONS) $(DESIGN_NAME)

fit: map
	quartus_fit $(QUARTUS_OPTIONS) $(DESIGN_NAME)

asm: fit
	quartus_asm $(QUARTUS_OPTIONS) $(DESIGN_NAME)

sta: asm
	quartus_sta $(QUARTUS_OPTIONS) $(DESIGN_NAME)

clean:
	rm -rf *.* db incremental_db
"""

    CONFIG_MK_TEMPLATE = """#Auto generated by Edalize

DESIGN_NAME := {design_name}
QUARTUS_OPTIONS := {quartus_options}

qsys:"""

    QSYS_TEMPLATE = """
	ip-generate \
--project-directory={src_dir} \
--output-directory={dst_dir} \
--report-file=bsf:{dst_dir}/{name}.bsf \
--system-info=DEVICE_FAMILY="{family}" \
--system-info=DEVICE={device} \
--component-file={src_dir}/{name}.qsys
	ip-generate \
--project-directory={src_dir} \
--output-directory={dst_dir}/synthesis \
--file-set=QUARTUS_SYNTH \
--report-file=sopcinfo:{dst_dir}/{name}.sopcinfo \
--report-file=html:{dst_dir}/{name}.html \
--report-file=qip:{dst_dir}/{name}.qip \
--report-file=cmp:{dst_dir}/{name}.cmp \
--report-file=svd \
--system-info=DEVICE_FAMILY="{family}" \
--system-info=DEVICE={device} \
--component-file={src_dir}/{name}.qsys \
--language=VERILOG
"""

    def configure_main(self):
        for i in ['family', 'device']:
            if not i in self.tool_options:
                raise RuntimeError("Missing required option '{}'".format(i))

        with open(os.path.join(self.work_root, self.name.replace('.', '_')+'.tcl'), 'w') as tcl_file:
            s = """project_new {} -overwrite
set_global_assignment -name FAMILY "{}"
set_global_assignment -name DEVICE {}
set_global_assignment -name TOP_LEVEL_ENTITY {}
"""

            tcl_file.write(s.format(self.name.replace('.', '_'),
                                    self.tool_options['family'],
                                    self.tool_options['device'],
                                    self.toplevel))

            for key, value in self.vlogparam.items():
                tcl_file.write("set_parameter -name {} {}\n".format(key, self._param_value_str(value)))
            for key, value in self.vlogdefine.items():
                tcl_file.write('set_global_assignment -name VERILOG_MACRO "{}={}"\n'.format(key, self._param_value_str(value)))

            (src_files, incdirs) = self._get_fileset_files()

            qsys_files = []
            for f in src_files:
                if f.file_type in ["verilogSource",
                                   "verilogSource-95",
                                   "verilogSource-2001",
                                   "verilogSource-2005"]:
                    _type = 'VERILOG_FILE'
                elif f.file_type in ["systemVerilogSource",
                                     "systemVerilogSource-3.0",
                                     "systemVerilogSource-3.1",
                                     "systemVerilogSource-3.1a"]:
                    _type = 'SYSTEMVERILOG_FILE'
                elif f.file_type in ['vhdlSource',
                                     'vhdlSource-87',
                                     'vhdlSource-93',
                                     'vhdlSource-2008']:
                    _type = 'VHDL_FILE'
                elif f.file_type in ['QIP']:
                    _type = 'QIP_FILE'
                elif f.file_type in ['QSYS']:
                    #Each qsys file will be run through ip-generate, which will
                    #generate a qip file with the same name as the qsys file
                    #The qip will will be stored in work_root/qsys/name/name.qip
                    #Therefore we replace the qsys_file with the qip file here
                    _src_dir = os.path.dirname(f.name)
                    _name = os.path.basename(f.name).split('.qsys')[0]
                    _dst_dir = os.path.join('qsys', _name)

                    qsys_files.append((_src_dir, _dst_dir, _name))

                    f.name = os.path.join(_dst_dir, _name+'.qip')
                    _type = 'QIP_FILE'
                elif f.file_type in ['SDC']:
                    _type = 'SDC_FILE'
                elif f.file_type in ['tclSource']:
                    tcl_file.write("source {}\n".format(f.name.replace('\\', '/')))
                    _type = None
                elif f.file_type in ['user']:
                    _type = None
                else:
                    _type = None
                    _s = "{} has unknown file type '{}'"
                    logger.warning(_s.format(f.name,
                                             f.file_type))
 
                if _type:
                    if f.logical_name != "":            
                        _s = "set_global_assignment -name {} -library {} {}\n"
                        tcl_file.write(_s.format(_type,
                                                 f.logical_name,
                                                 f.name.replace('\\', '/')))
                    else:
                        _s = "set_global_assignment -name {} {}\n"
                        tcl_file.write(_s.format(_type,
                                                 f.name.replace('\\', '/')))                         

            for include_dir in incdirs:
                tcl_file.write("set_global_assignment -name SEARCH_PATH " + include_dir.replace('\\', '/') + '\n')

        with open(os.path.join(self.work_root, 'Makefile'), 'w') as makefile:
            makefile.write(self.MAKEFILE_TEMPLATE)

        with open(os.path.join(self.work_root, 'config.mk'), 'w') as config_mk:
            quartus_options = ' '.join(self.tool_options.get('quartus_options', []))
            config_mk.write(self.CONFIG_MK_TEMPLATE.format(
                design_name     = self.name.replace('.', '_'),
                quartus_options = quartus_options))
            for qsys_file in qsys_files:
                config_mk.write(self.QSYS_TEMPLATE.format(
                    src_dir = qsys_file[0],
                    dst_dir = qsys_file[1],
                    name    = qsys_file[2],
                    family  = self.tool_options['family'],
                    device  = self.tool_options['device']))

    def run(self, remaining):
        args = ['--mode=jtag']
        args += remaining
        args += ['-o']
        args += ['p;' + self.name.replace('.', '_') + '.sof']
        self._run_tool('quartus_pgm', args)



import tempfile
import shutil
import os.path
import logging
import subprocess

from argparse import Namespace

from fusesoc.config import Config

from test_common import cores_root, cache_root, library_root

build_root = 'test_build_root'

EXAMPLE_CONFIG = """
[main]
build_root = {build_root}
cache_root = {cache_root}
library_root = {library_root}

[library.test_lib]
location = {cores_root}
auto-sync = {auto_sync}
sync-uri = {sync_uri}
sync-type = {sync_type}
"""

sync_uri = 'https://github.com/fusesoc/fusesoc-cores'

def test_library_location():
    from fusesoc.main import _get_core, init_coremanager

    tcf = tempfile.TemporaryFile(mode="w+")
    tcf.write(EXAMPLE_CONFIG.format(
            build_root = build_root,
            cache_root = cache_root,
            cores_root = cores_root,
            library_root = library_root,
            auto_sync = 'false',
            sync_uri = sync_uri,
            sync_type = 'git'
            )
        )
    tcf.seek(0)

    conf = Config(file=tcf)
    cm = init_coremanager(conf, [])

    _get_core(cm, 'mor1kx-generic')
    _get_core(cm, 'atlys')

def test_library_add(caplog):
    from fusesoc.main import add_library
    from fusesoc.coremanager import CoreManager

    # Set up safe environment variables for library location
    os.environ['XDG_DATA_HOME']  =  library_root

    tcf = tempfile.NamedTemporaryFile(mode="w+")
    clone_target = "tests/test_libraries"

    conf = Config(file=tcf)
    conf.library_root = library_root
    cm = CoreManager(conf)

    args = Namespace()

    args.name = 'fusesoc-cores'
    args.location = clone_target
    args.config = tcf
    args.no_auto_sync = False
    vars(args)['sync-uri'] = sync_uri

    add_library(cm, args)

    expected = """[library.fusesoc-cores]
sync-uri = https://github.com/fusesoc/fusesoc-cores
location = {}""".format(os.path.abspath(clone_target))

    tcf.seek(0)
    result = tcf.read().strip()

    assert expected == result

    tcf.close()
    shutil.rmtree(clone_target)
    tcf = tempfile.NamedTemporaryFile(mode="w+")

    args.config = tcf
    args.location = None
    vars(args)['sync-type'] = 'git'

    expected = """[library.fusesoc-cores]
sync-uri = https://github.com/fusesoc/fusesoc-cores
sync-type = git"""

    add_library(cm, args)

    tcf.seek(0)
    result = tcf.read().strip()

    assert expected == result

    tcf.close()
    shutil.rmtree(os.path.join(library_root, "fusesoc"))
    tcf = tempfile.NamedTemporaryFile(mode="w+")

    args.config = tcf
    vars(args)['sync-type'] = 'local'
    vars(args)['sync-uri'] = 'tests/capi2_cores'
    args.location = None

    with caplog.at_level(logging.INFO):
        add_library(cm, args)

    assert "Interpreting sync-uri 'tests/capi2_cores' as location for local provider." in caplog.text

def test_library_update(caplog):
    from fusesoc.main import update, init_coremanager, init_logging

    clone_target = tempfile.mkdtemp()

    try:
        subprocess.call(['git', 'clone', sync_uri, clone_target])

        tcf = tempfile.TemporaryFile(mode="w+")
        tcf.write(EXAMPLE_CONFIG.format(
                build_root = build_root,
                cache_root = cache_root,
                cores_root = clone_target,
                library_root = library_root,
                auto_sync = 'false',
                sync_uri = sync_uri,
                sync_type = 'git'
                )
            )
        tcf.seek(0)

        conf = Config(file=tcf)

        args = Namespace()

        init_logging(False, False)
        cm = init_coremanager(conf, [])

        # TODO find a better way to set up these defaults
        args.libraries = []

        with caplog.at_level(logging.INFO):
            update(cm, args)

        assert not "Updating 'test_lib'" in caplog.text

        caplog.clear()

        args.libraries = ['test_lib']

        with caplog.at_level(logging.INFO):
            update(cm, args)

        assert "Updating 'test_lib'" in caplog.text

        caplog.clear()

        args.libraries = []
        conf.libraries['test_lib']['auto-sync'] = True

        with caplog.at_level(logging.INFO):
            update(cm, args)

        assert "Updating 'test_lib'" in caplog.text

        caplog.clear()

        tcf.close()
        tcf = tempfile.TemporaryFile(mode="w+")
        tcf.write(EXAMPLE_CONFIG.format(
                build_root = build_root,
                cache_root = cache_root,
                cores_root = clone_target,
                library_root = library_root,
                auto_sync = 'true',
                sync_uri = sync_uri,
                sync_type = 'local'
                )
            )
        tcf.seek(0)

        conf = Config(file=tcf)

        args.libraries = []

        with caplog.at_level(logging.INFO):
            update(cm, args)

        assert "Updating 'test_lib'" in caplog.text

    finally:
        shutil.rmtree(clone_target)



import logging
import os
import shutil
from fusesoc.provider.provider import Provider
from fusesoc.utils import Launcher

logger = logging.getLogger(__name__)

class Coregen(Provider):

    def _checkout(self, local_dir):
        script_file  = self.config.get('script_file')
        project_file = self.config.get('project_file')
        extra_files  = self.config.get('extra_files')
        logger.info("Using Coregen to generate project " + project_file)
        if not os.path.isdir(local_dir):
            os.makedirs(local_dir)
        src_files = [script_file, project_file]
        if extra_files:
            src_files += extra_files.split()

        for f in src_files:
            f_src = os.path.join(self.core_root, f)
            f_dst = os.path.join(local_dir, f)
            if os.path.exists(f_src):
                d_dst = os.path.dirname(f_dst)
                if not os.path.exists(d_dst):
                    os.makedirs(d_dst)
                shutil.copyfile(f_src, f_dst)
            else:
                logger.error('Cannot find file %s' % f_src)
        args = ['-r',
                '-b', script_file,
                '-p', project_file]
        Launcher('coregen', args, cwd=local_dir).run()



import pytest


def test_spyglass_defaults():
    """ Test if the SpyGlass backend picks up the tool defaults """
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir = os.path.join(tests_dir, __name__, 'defaults')
    paramtypes = ['vlogdefine', 'vlogparam']
    name = 'test_spyglass_0'
    tool = 'spyglass'
    tool_options = {}

    (backend, args, work_root) = setup_backend(
        paramtypes, name, tool, tool_options)
    backend.configure(args)

    compare_files(ref_dir, work_root, [
        'Makefile',
        'spyglass-run-design_read.tcl',
        'spyglass-run-lint_lint_rtl.tcl',
        name + '.prj',
    ])

    backend.build()
    compare_files(ref_dir, work_root, [
        'spyglass.cmd',
    ])


def test_spyglass_tooloptions():
    """ Test passing tool options to the Spyglass backend """
    import os
    import shutil
    from edalize_common import compare_files, setup_backend, tests_dir

    ref_dir = os.path.join(tests_dir, __name__, 'tooloptions')
    paramtypes = ['vlogdefine', 'vlogparam']
    name = 'test_spyglass_0'
    tool = 'spyglass'
    tool_options = {
        'methodology': 'GuideWare/latest/block/rtl_somethingelse',
        'goals': ['lint/lint_rtl', 'some/othergoal'],
        'spyglass_options': ['handlememory yes'],
    }

    (backend, args, work_root) = setup_backend(
        paramtypes, name, tool, tool_options)
    backend.configure(args)

    compare_files(ref_dir, work_root, [
        'Makefile',
        'spyglass-run-design_read.tcl',
        'spyglass-run-lint_lint_rtl.tcl',
        'spyglass-run-some_othergoal.tcl',
        name + '.prj',
    ])

    backend.build()
    compare_files(ref_dir, work_root, [
        'spyglass.cmd',
    ])



import logging
import os.path
import sys
import tarfile
from fusesoc.provider.provider import Provider

logger = logging.getLogger(__name__)

if sys.version_info[0] >= 3:
    import urllib.request as urllib
    from urllib.error import URLError
else:
    import urllib
    from urllib2 import URLError

URL = 'https://github.com/{user}/{repo}/archive/{version}.tar.gz'

class Github(Provider):

    def _checkout(self, local_dir):
        user   = self.config.get('user')
        repo   = self.config.get('repo')

        version = self.config.get('version', 'master')

        #TODO : Sanitize URL
        url = URL.format(user=user,
                         repo=repo,
                         version=version)
        logger.info("Downloading {}/{} from github".format(user,
                                                       repo))
        try:
            (filename, headers) = urllib.urlretrieve(url)
        except URLError as e:
            raise RuntimeError("Failed to download '{}'. '{}'".format(url, e.reason))
        t = tarfile.open(filename)
        (cache_root, core) = os.path.split(local_dir)

        #Ugly hack to get the first part of the directory name of the extracted files
        tmp = t.getnames()[0]
        t.extractall(cache_root)
        os.rename(os.path.join(cache_root, tmp),
                  os.path.join(cache_root, core))
